@@ -1,11 +1,11 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
-;;; Copyright © 2014, 2018, 2020, 2021 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2014, 2018, 2020, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2017, 2018, 2019, 2020 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2017, 2018, 2020 Johri van Eerd <vaneerd.johri@gmail.com>
 ;;; Copyright © 2018 Filip Toman <filip.toman@verum.com>
-;;; Copyright © 2019, 2020 Paul Hoogendijk <paul@dezyne.org>
+;;; Copyright © 2019, 2020, 2022 Paul Hoogendijk <paul@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -68,7 +68,6 @@
            ast:full-scope
            ast:get-model
            ast:graph-cyclic?
-           ast:id-path
            ast:imperative?
            ast:import*
            ast:imported?
@@ -82,7 +81,9 @@
            ast:literal-false?
            ast:literal-true?
            ast:location
+           ast:location->string
            ast:name
+           ast:normalize
            ast:optional?
            ast:other-direction
            ast:other-end-point
@@ -100,7 +101,7 @@
            ast:rescope
            ast:req-events
            ast:requires-port*
-           ast:required+async
+           ast:requires+async-port*
            ast:required-in-triggers
            ast:required-out-triggers
            ast:rescope
@@ -122,6 +123,7 @@
            ast:provides?
            ast:requires?
            ast:wildcard?
+           ast:blocking?
            ast:external?
            ast:injected?
            ast:type
@@ -295,6 +297,9 @@
 (define-method (ast:requires? (o <trigger>))
   (and (.port.name o) ((compose ast:requires? .port) o)))
 
+(define-method (ast:requires? (o <action>))
+  (and (.port.name o) ((compose ast:requires? .port) o)))
+
 (define-method (ast:empty-namespace? (o <string>))
   (equal? o "/"))
 
@@ -304,11 +309,17 @@
 (define-method (ast:wildcard? (o <boolean>))
   #f)
 
+(define-method (ast:blocking? (o <port>))
+  (and (.blocking? o) o))
+
+(define-method (ast:blocking? (o <action>))
+  (and=> (.port o) ast:blocking?))
+
 (define-method (ast:external? (o <port>))
-  (and (.external o) o))
+  (and (.external? o) o))
 
 (define-method (ast:injected? (o <port>))
-  (and (.injected o) o))
+  (and (.injected? o) o))
 
 (define-method (ast:other-direction (o <event>))
   (assoc-ref `((in . out)
@@ -321,18 +332,28 @@
 (define-method (ast:injected-port* (o <component-model>))
   (filter ast:injected? (ast:port* o)))
 
-(define-method (ast:provides-port* (o <port>))
-  (ast:provides-port* (parent o <component-model>)))
+(define-method (ast:injected-port* (o <trigger>))
+  (ast:injected-port* (parent o <component-model>)))
 
 (define-method (ast:provides-port* (o <component-model>))
   (filter ast:provides? (ast:port* o)))
 
+(define-method (ast:provides-port* (o <port>))
+  (ast:provides-port* (parent o <component-model>)))
+
 (define-method (ast:requires-port* (o <component-model>))
   (filter ast:requires? (ast:port* o)))
 
-(define-method (ast:required+async (o <component-model>))
+(define-method (ast:requires-port* (o <port>))
+  (ast:requires-port* (parent o <component-model>)))
+
+(define-method (ast:requires+async-port* (o <component-model>))
   (append (ast:requires-port* o) (ast:async-port* o)))
 
+(define-method (ast:direction (o <port>))
+  (if (ast:provides? o) 'in
+      'out))
+
 (define-method (ast:direction (o <trigger>))
   (.direction (.event o)))
 
@@ -557,11 +578,10 @@
 (define-method (ast:path (o <ast>) stop?)
   (unfold stop? identity .parent o))
 
-(define-method (ast:id-path (o <ast>))
-  (map .id (ast:path o (negate identity))))
-
 (define-method (ast:eq? (a <ast>) (b <ast>))
-  (equal? (ast:id-path a) (ast:id-path b)))
+  (or (eq? a b)
+      (and (eq? (.node a) (.node b))
+           (ast:eq? (.parent a) (.parent b)))))
 
 (define-method (ast:eq? (a <ast>) b)
   #f)
@@ -741,13 +761,19 @@
   "Return all event types used in COMPONENT."
   (delete-duplicates (append-map ast:return-types (filter-map ast:type (ast:provides-port* component))) ast:eq?))
 
-(define-method (ast:return-values (o <event>))
+(define-method (ast:return-values (o <event>) void)
   (let ((type ((compose .type .signature) o)))
-    (cond ((as type <void>) '())
+    (cond ((as type <void>) void)
           ((as type <enum>) (map (cut make <enum-literal> #:type.name (.name type) #:field <>) (ast:field* type)))
           ((as type <bool>) (map (cut make <literal> #:value <>) '("true" "false")))
           ((as type <int>) (map (cut make <literal> #:value <>) (iota (1+ (- (.to (.range type)) (.from (.range type)))) (.from (.range type))))))))
 
+(define-method (ast:return-values (o <event>))
+  (ast:return-values o '()))
+
+(define-method (ast:return-values (o <port>))
+  (append-map (cute ast:return-values <> '("return")) (ast:in-event* o)))
+
 (define-method (ast:return-values (o <action>))
   (ast:return-values (.event o)))
 
@@ -765,6 +791,14 @@
 
 (define-method (ast:location o) #f)
 
+(define (ast:location->string o)
+  (let ((location (ast:location o)))
+    (and location
+         (format #f "~a:~a:~a"
+                 (.file-name location)
+                 (.line location)
+                 (.column location)))))
+
 (define-method (ast:base-name (o <ast>))
   (basename (ast:source-file o) ".dzn"))
 
@@ -833,7 +867,7 @@
                    (else (loop (cdr bindings)))))))))
 
 (define-method (ast:other-end-point-injected (system <system>) (o <port>))
-  ;; pre: (and (.injected o) (not "o is directly bound"))
+  ;; pre: (and (.injected? o) (not "o is directly bound"))
   (let loop ((bindings (ast:binding* system)))
     (and (pair? bindings)
          (let* ((binding (car bindings))
@@ -899,7 +933,8 @@
   '("data"))
 
 (define-method (ast:full-name (o <int>))
-  '("int"))
+  (if (ast:name o) (next-method)
+      '("int")))
 
 (define-method (ast:full-name (o <void>))
   '("void"))
@@ -1051,8 +1086,11 @@
 
 (define-method (ast:lookup-n (o <scope>) (name <scope.name>))
   (let ((ids (.ids name)))
-    (if (null? (cdr ids)) (if (ast:has-equal-name (car ids) o) (list o)
-                              (ast:lookup-n o (car ids)))
+    (if (null? (cdr ids))
+        (let ((down (ast:lookdown o name)))
+          (if (pair? down) down
+              (if (ast:has-equal-name (car ids) o) (list o)
+                  (ast:lookup-n o (car ids)))))
         (let* ((first (car ids))
                (first-scopes (ast:lookup-n o first)))
           (if (null? first-scopes) '()
@@ -1205,33 +1243,67 @@
 (define-method (.function (o <call>))
   (and (.function.name o) (ast:lookup o (.function.name o))))
 
-(define-method (ast:lookup-var (o <ast>) name)
+(define-method (ast:lookup-variable (o <ast>) name statements)
   (define (name? o) (and (equal? (.name o) name) o))
   (match o
-    (($ <behavior>) (find name? (ast:variable* o)))
-    ((? (is? <compound>)) (or (find name? (filter (is? <variable>) (ast:statement* o))) (ast:lookup-var (.parent o) name)))
-    (($ <function>) (or (find name? ((compose ast:formal* .signature) o)) (ast:lookup-var (.parent o) name)))
-    (($ <formal>) (and (equal? (.name o) name) o))
-    (($ <formal-binding>) (and (equal? (.name o) name) o))
-    (($ <on>) (or (find (cut ast:lookup-var <> name) (append-map ast:formal* (ast:trigger* o))) (ast:lookup-var (.parent o) name)))
-    (($ <variable>) (name? o))
-    ((? (lambda (o) (is-a? (.parent o) <variable>))) (ast:lookup-var ((compose .parent .parent) o) name))
-    (_ (ast:lookup-var (.parent o) name))))
+    (($ <behavior>)
+     (find name? (ast:variable* o)))
+    ((? (is? <compound>))
+     (or
+      (find name? (filter (is? <variable>) statements))
+      (and (is-a? (.parent o) <compound>)
+           (ast:lookup-variable (.parent o) name (ast:statement-prefix o)))
+      (ast:lookup-variable (.parent o) name statements)))
+    ((? (is? <if>))
+     (or
+      (and (is-a? (.parent o) <compound>)
+              (ast:lookup-variable (.parent o) name (ast:statement-prefix o)))
+      (ast:lookup-variable (.parent o) name statements)))
+    (($ <function>)
+     (or (find name? ((compose ast:formal* .signature) o))
+         (ast:lookup-variable (.parent o) name statements)))
+    (($ <formal>)
+     (name? o))
+    (($ <formal-binding>)
+     (name? o))
+    (($ <on>)
+     (or (find (cute ast:lookup-variable <> name statements)
+               (append-map ast:formal* (ast:trigger* o)))
+         (ast:lookup-variable (.parent o) name statements)))
+    (($ <variable>)
+     (name? o))
+    ((? (lambda (o) (is-a? (.parent o) <variable>)))
+     (ast:lookup-variable ((compose .parent .parent) o) name statements))
+    (_
+     (ast:lookup-variable (.parent o) name statements))))
 
-(define-method (ast:lookup-var (o <boolean>) name)
+(define (ast:lookup-variable- root o name)
+  (ast:lookup-variable o name (ast:statement-prefix o)))
+
+(define-method (ast:lookup-variable (o <ast>) name)
+  ((ast:pure-funcq ast:lookup-variable-) (parent o <root>) o name))
+
+(define-method (ast:statement-prefix (o <ast>))
+  (let ((compound (parent (if (is-a? o <compound>) (.parent o) o) <compound>)))
+    (if (not compound) '()
+        (let* ((statements (ast:statement* compound))
+               (path (ast:path o)))
+          (take-while (negate (cute member <> path ast:eq?)) statements)))))
+
+(define-method (ast:lookup-variable (o <boolean>) name)
   #f)
 
 (define-method (.variable (o <assign>))
-  (and=> (.variable.name o) (cut ast:lookup-var o <>)))
+  (and=> (.variable.name o) (cut ast:lookup-variable o <>)))
 
 (define-method (.variable (o <field-test>))
-  (and=> (.variable.name o) (cut ast:lookup-var o <>)))
+  (and=> (.variable.name o) (cut ast:lookup-variable o <>)))
 
 (define-method (.variable (o <formal-binding>))
-  (and=> (.variable.name o) (cut ast:lookup-var (.parent o) <>)))
+  (and=> (.variable.name o) (cut ast:lookup-variable (.parent o) <>)))
 
 (define-method (.variable (o <var>))
-  (and=> (.name o) (cut ast:lookup-var o <>)))
+  (and=> (.name o) (cut ast:lookup-variable o <>)))
 
 (define-method (.type (o <argument>))
   (ast:lookup o (.type.name o)))
@@ -1277,7 +1349,9 @@
         (ast:lookup o name))))
 
 (define-method (.type (o <port>))
-  (ast:lookup o (.type.name o)))
+  (let ((component (parent o <component-model>)))
+    (and component
+         (ast:lookup (.parent component) (.type.name o)))))
 
 (define-method (.type (o <signature>))
   (ast:lookup o (.type.name o)))
@@ -1285,6 +1359,22 @@
 (define-method (.type (o <variable>))
   (ast:lookup o (.type.name o)))
 
+(define-method (ast:normalize (o <binding>))
+  "Bring binding O in canonical form, i.e., ordering from a system's top
+to bottom."
+  (let* ((left (.left o))
+         (right (.right o))
+         (left-instance (.instance left))
+         (right-instance (.instance right))
+         (left-provides? (and=> (.port left) ast:provides?))
+         (right-provides? (and=> (.port right) ast:provides?))
+         (canonical?
+          (or (and (not left-instance) right-instance left-provides?)
+              (and left-instance right-instance (not left-provides?))
+              (and left-instance (not right-instance) (not left-provides?)))))
+    (if canonical? o
+        (clone o #:left right #:right left))))
+
 (define-method (topological-sort (dag <list>) key)
 "Sort DAG topologically using function KEY, where DAG looks like
 
