@@ -1,6 +1,6 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2019, 2020, 2021 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2019, 2020 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2019, 2020, 2021 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2021 Paul Hoogendijk <paul@dezyne.org>
@@ -42,7 +42,6 @@
   #:use-module (dzn command-line)
   #:use-module (dzn misc)
   #:use-module (dzn parse)
-  #:use-module (dzn parse silence)
   #:export (parse-tree->ast
             annotate-ast))
 
@@ -169,10 +168,7 @@
              #:events (make <events-node> #:elements events))))
 
         (('interface name (and ('behavior x ...) behavior))
-         (let* ((behavior (set-recursive (helper behavior)))
-                (behavior (and behavior
-                                (.node ((mark-silent)
-                                        (make <behavior> #:node behavior))))))
+         (let ((behavior (set-recursive (helper behavior))))
            (make <interface-node>
              #:name (helper name)
              #:behavior behavior)))
@@ -180,10 +176,7 @@
         (('interface name types-and-events behavior)
          (let* ((types-and-events (helper types-and-events))
                 (types events (partition (is? <type-node>) types-and-events))
-                (behavior (set-recursive (helper behavior)))
-                (behavior (and behavior
-                                (.node ((mark-silent)
-                                        (make <behavior> #:node behavior))))))
+                (behavior (set-recursive (helper behavior))))
            (make <interface-node>
              #:name (helper name)
              #:types (make <types-node> #:elements types)
@@ -282,8 +275,9 @@
              #:name (helper name)
              #:type.name type
              #:direction direction
-             #:external (and=> (assq 'external qualifiers) helper)
-             #:injected (and=> (assq 'injected qualifiers) helper))))
+             #:blocking? (and=> (assq 'blocking-q qualifiers) helper)
+             #:external? (and=> (assq 'external qualifiers) helper)
+             #:injected? (and=> (assq 'injected qualifiers) helper))))
 
         (('port direction ('port-qualifiers qualifiers ...) type formals name)
          (let* ((direction (helper direction))
@@ -296,11 +290,13 @@
              #:type.name type
              #:formals formals
              #:direction direction
-             #:external (and=> (assq 'external qualifiers) helper)
-             #:injected (and=> (assq 'injected qualifiers) helper))))
+             #:blocking? (and=> (assq 'blocking-q qualifiers) helper)
+             #:external? (and=> (assq 'external qualifiers) helper)
+             #:injected? (and=> (assq 'injected qualifiers) helper))))
 
         (('provides) 'provides)
         (('requires) 'requires)
+        (('blocking-q) 'blocking)
         (('external) 'external)
         (('injected) 'injected)
 
