@@ -70,7 +70,7 @@
     (string-join alphabet ",")))
 
 (define (component-taus model)
-  (let ((ports (ast:required+async model)))
+  (let ((ports (ast:requires+async-port* model)))
     (string-join (map makreel:.name ports) ",")))
 
 (define (component-exclude-taus model)
@@ -137,7 +137,8 @@ actions."
   (define (trigger->event trigger)
     (let ((port (.port trigger)))
       (string-append (makreel:.name (.port trigger))
-                     (if (ast:async? trigger) ".qout." ".")
+                     (if (or (ast:async? trigger) (ast:out? trigger)) ".qout."
+                         ".")
                      (.event.name trigger))))
   (let* ((triggers (ast:in-triggers component))
          (alphabet (map trigger->event triggers)))
@@ -150,6 +151,7 @@ actions."
                (and (not (member event '("inevitable" "optional" "tau" ;;"<illegal>" "<declarative-illegal>"
                                          )))
                     (not (string-contains event ".qout."))
+                    (not (string-contains event ".<blocking>"))
                     (not (find (cute string-suffix? <> event) '(".optional" ".inevitable")))))
              (string-split trace #\;))
      "\n")))
@@ -531,19 +533,16 @@ init for MODEL unless INIT."
 
 (define (mcrl2:verify-interface-asserts model root)
   (let* ((model-name (makreel:unticked-dotted-name model))
-         (skip-nondet? (command-line:get 'no-interface-determinism))
          (result (string-append
                   (verify-pipeline "verify-interface" root model)
-                  (if skip-nondet? ""
-                      (verify-pipeline "verify-interface-nondet" root model))))
+                  (verify-pipeline "verify-interface-nondet" root model)))
          (result (result-split result)))
     (define (report-assert assert)
       (report assert #f (get-trace assert result) 'interface model-name))
     (reduce-or (command-line:get 'all)
                (list (cute report-assert 'deadlock)
                      (cute report-assert 'livelock)
-                     (if skip-nondet? (const #f)
-                         (cute report-assert 'deterministic))))))
+                     (cute report-assert 'deterministic)))))
 
 (define (mcrl2:verify-compliance root model)
   (let* ((output status (verify-pipeline "verify-compliance" root model))
