@@ -1,8 +1,8 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2018, 2019, 2020, 2021 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2018, 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2018, 2019 Rob Wieringa <rma.wieringa@gmail.com>
-;;; Copyright © 2020, 2021 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2020, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -47,7 +47,6 @@
             display-trails
             label->string
             set-trigger-locations
-            step->location
             trace-equal?
             trace->trail
             trace->string-trail
@@ -85,9 +84,13 @@
     (map strip-sut-prefix trail)))
 
 (define (trace-equal? a b)
-  (and (rtc-program-counter-equal? (car a) (car b))
-       (equal? (trace->string-trail a)
-               (trace->string-trail b))))
+  (let ((pc-a (car a))
+        (pc-b (car b)))
+   (and (rtc-program-counter-equal? pc-a pc-b)
+        (ast:eq? (and=> (.previous pc-a) .statement)
+                 (and=> (.previous pc-b) .statement))
+        (equal? (trace->string-trail a)
+                (trace->string-trail b)))))
 
 (define (debug:lts->alist pc->state-number lts)
   (define (entry->pair from pc+traces)
@@ -115,9 +118,11 @@
   (pc-event? (.instance pc) (.trigger pc)))
 
 (define-method (pc-event? (o <runtime:port>) (trigger <trigger>))
-  (and (eq? o (%sut))
-       (or (%modeling?)
-           (not (ast:modeling? trigger)))))
+  (let ((modeling? (ast:modeling? trigger)))
+    (or (and (eq? o (%sut))
+             (not modeling?))
+        (and (%modeling?)
+             modeling?))))
 
 (define-method (pc-event? (o <runtime:component>) (trigger <trigger>))
   (let ((port (.port trigger)))
@@ -203,7 +208,10 @@
   (pc->event (.instance pc) o (.trigger pc)))
 
 (define-method (pc->event (o <runtime:port>) (compound <initial-compound>) (trigger <trigger>))
-  (cons trigger (format #f "~a" (.event.name trigger))))
+  (cons trigger
+        (if (is-a? (%sut) <runtime:port>)
+            (format #f "~a" (.event.name trigger))
+            (format #f "~a.~a" (.name (.ast o)) (.event.name trigger)))))
 
 (define-method (pc->event (o <runtime:component>) (compound <initial-compound>) (trigger <trigger>))
   (cons trigger
@@ -214,7 +222,10 @@
 (define-method (pc->event (o <runtime:port>) (action <action>))
   (cons action
         (if (ast:out? action)
-            (format #f "~a~a" (or (and=> (trace-name o) (cut format #f "~a." <>)) "") (trigger->string action))
+            (format #f "~a~a" (or (and=> (trace-name o)
+                                         (cute format #f "~a." <>))
+                                  "")
+                    (trigger->string action))
             (format #f "~a" (trigger->string action)))))
 
 (define-method (pc->event (o <runtime:component>) (action <action>))
@@ -222,10 +233,13 @@
         (let* ((port (.port action))
                (r:port (runtime:port o port))
                (r:other-port (runtime:other-port r:port)))
-          (if (ast:injected? port)
-              (format #f "~a.~a" (.name (.ast r:other-port)) (.event.name action))
-              (let ((trigger (action->trigger r:other-port action)))
-                (format #f "~a.~a" (trace-name r:other-port) (trigger->string trigger)))))))
+          (cond
+           ((ast:injected? port)
+            (format #f "~a.~a" (trace-name r:other-port) (.event.name action)))
+           (else
+            (let* ((trigger (action->trigger r:other-port action))
+                   (trigger (trigger->string trigger)))
+              (format #f "~a.~a" (trace-name r:other-port) trigger)))))))
 
 (define-method (pc->event (o <runtime:component>) (q-out <q-out>))
   (let ((trigger (.trigger q-out)))
@@ -248,7 +262,7 @@
             (if (eq? r:port r:other-port) ;The other port is injected,
                                           ;no chance to find our way back
                                           ;and use ast:injected?
-                (format #f "~a.~a" (.name port) (.event.name return))
+                (format #f "~a.~a" (trace-name r:port) (.event.name return))
                 (let ((value (.event.name return)))
                   (format #f "~a.~a" (trace-name r:other-port) value)))))))
 
@@ -603,7 +617,10 @@ Add (synthesize) missing PCs for <q-in>, <q-out> and <trigger-return>."
                    (r:port (runtime:port pc-instance port))
                    (r:other-port (and r:port (runtime:other-port r:port)))
                    (q-trigger (clone trigger #:port.name #f))
-                   (on (find (compose (is? <on>) .statement) result))
+                   (on (find (conjoin
+                              (compose (is? <on>) .statement)
+                              (compose (cute eq? <> pc-instance) .instance))
+                             result))
                    (on (and on (.statement on)))
                    (location (if on (.location on) (.location statement)))
                    (pc (clone pc #:statement (clone statement #:location location)))
@@ -633,11 +650,11 @@ Add (synthesize) missing PCs for <q-in>, <q-out> and <trigger-return>."
                    (r:port (if (is-a? pc-instance <runtime:port>) pc-instance
                                (runtime:port pc-instance port)))
                    (r:other-port (runtime:other-port r:port))
-                   (interface (and port (.type port)))
+                   (interface (if port (.type port) (parent statement <interface>)))
                    (component (.type (.ast next-instance)))
-                   (port-name (if (eq? r:port r:other-port) (injected-port-name component interface)
+                   (port-name (if (or (not r:other-port) (eq? r:port r:other-port)) (injected-port-name component interface)
                                   (.name (.ast r:other-port))))
-                   (action (any (cute action-matches? (.container r:other-port) <>) trace))
+                   (action (and r:other-port (any (cute action-matches? (.container r:other-port) <>) trace)))
                    (return (clone statement
                                   #:port.name port-name
                                   #:location (.location (or action statement))))
@@ -669,13 +686,8 @@ the location of the executed <on>-statement."
                (pc (if (not trigger) pc (clone pc #:trigger trigger))))
           (cons pc (loop (cdr trace)))))))
 
-(define (step->location o)
-  (let ((location (ast:location o)))
-    (and location
-         (format #f "~a:~a:~a: "
-                 (.file-name location)
-                 (.line location)
-                 (.column location)))))
+(define (location-prefix o)
+  (format #f "~a: " (or (and=> o ast:location->string) "<unknown-file>")))
 
 (define* (display-trace trace #:key locations? state? verbose?)
   "Write TRACE as split-arrows trace to stdout.  When LOCATIONS?,
@@ -690,7 +702,7 @@ intermediate steps such as assignments, function calls, replies,
       (match-lambda*
         (((pc ast . string) i)
          (when (and ast locations?)
-           (let ((location (step->location ast)))
+           (let ((location (location-prefix ast)))
              (when location
                (display location))))
          (write-line string)
@@ -733,28 +745,24 @@ intermediate steps such as assignments, function calls, replies,
               (port-acceptances (and port-acceptances
                                      (ast:acceptance* port-acceptances)))
               (port-acceptance (and (pair? port-acceptances) (car port-acceptances)))
-              (location (or (step->location (or component-acceptance
-                                                port-acceptance))
-                            "<unknown-file>:"))
+              (location (location-prefix (or component-acceptance
+                                             port-acceptance)))
               (message (.message status)))
          (format #f "~aerror: ~a\n" location message)))
       (($ <fork-error>)
-       (let ((location (or (and=> (.ast status) step->location)
-                           "<unknown-file>:"))
+       (let ((location (and=> (.ast status) location-prefix))
              (message (.message status)))
          (format #f "~aerror: ~a\n" location message)))
       (($ <refusals-error>)
-       (let ((location (or (and=> (.ast status) step->location)
-                           "<unknown-file>:"))
+       (let ((location (and=> (.ast status) location-prefix))
              (message (.message status)))
          (format #f "~aerror: ~a\n" location message)))
       ((and (? (is? <determinism-error>)))
        (let* ((ast (or (.ast status)
                        (.behavior (runtime:%sut-model))))
               (locations (map (lambda (pc)
-                                (or (and=> (.ast (.status pc)) step->location)
-                                    (step->location ast)
-                                    "<unknown-file>:"))
+                                (or (and=> (.ast (.status pc)) location-prefix)
+                                    (location-prefix ast)))
                               pcs))
               (message (if (is-a? (parent ast <model>) <component>)
                            "component is non-deterministic due to overlapping guards"
@@ -763,34 +771,28 @@ intermediate steps such as assignments, function calls, replies,
           (map (cut format #f "~aerror: ~a\n" <> message) locations) "")))
       (($ <queue-full-error>)
        (let* ((model ((compose  ast:dotted-name .type .ast .instance) status))
-              (location (or (step->location (.ast status))
-                            "<unknown-file>:")))
+              (location (location-prefix (.ast status))))
          (format #f "~aerror: queue-full in component ~s\n" location model)))
       (($ <range-error>)
        (let* ((variable (.variable status))
               (name (.name variable))
               (type (.type variable))
               (value (.value status))
-              (location (or (step->location (.ast status))
-                            "<unknown-file>:")))
+              (location (location-prefix (.ast status))))
          (format #f "~aerror: range-error: invalid value `~a' for variable `~a'\n" location value name)))
       ((and (? (is? <missing-reply-error>)))
        (let* ((type (.type status))
-              (location (or (step->location (.ast status))
-                            "<unknown-file>:")))
+              (location (location-prefix (.ast status))))
          (format #f "~aerror: missing-reply: ~a reply expected\n" location (type-name type))))
       (($ <second-reply-error>)
        (let* ((previous (.previous status))
-              (location (or (step->location (.ast status))
-                            "<unknown-file>:")))
+              (location (location-prefix (.ast status))))
          (format #f "~aerror: second-reply\n" location)))
       ((and (? (is? <match-error>)) (= .ast ast))
-       (let ((location (or (step->location ast)
-                           "<unknown-file>:")))
+       (let ((location (location-prefix ast)))
          (format #f "~aerror: ~a\n" location (.input status))))
       ((and (? (is? <error>)) (= .ast ast) (= .message message))
-       (let ((location (or (step->location ast)
-                           "<unknown-file>:")))
+       (let ((location (location-prefix ast)))
          (format #f "~aerror: ~a\n" location message)))
       (($ <end-of-trail>) #f)
       (_ #f #f))))
@@ -806,9 +808,8 @@ intermediate steps such as assignments, function calls, replies,
               (port-acceptances (and port-acceptances
                                      (ast:acceptance* port-acceptances)))
               (port-acceptance (and (pair? port-acceptances) (car port-acceptances)))
-              (location (or (step->location (or component-acceptance
-                                                port-acceptance))
-                            "<unknown-file>:"))
+              (location (location-prefix (or component-acceptance
+                                             port-acceptance)))
               (acceptance (if component-acceptance (trigger->string component-acceptance)
                               "-"))
               (r:port (.port status))
@@ -818,25 +819,27 @@ intermediate steps such as assignments, function calls, replies,
          (string-join
           (append
            (if (not trigger) '()
-               (let ((location (step->location (.event trigger))))
-                 (list (format #f "~atrigger: ~a\n" location (trigger->string trigger)))))
-           (list (format #f "~acomponent accept: ~a\n" location acceptance))
+               (let ((location (location-prefix (.event trigger))))
+                 (if (not location) '()
+                     (list (format #f "~atrigger: ~a\n" location
+                                   (trigger->string trigger))))))
+           (list (format #f "~acomponent performs: ~a\n" location acceptance))
            (if (not port-acceptances)
                (let* ((interface (.type (.ast r:port)))
                       (ast (.behavior interface)))
-                 (list (format #f "~aport accept: -\n" (step->location ast))))
+                 (list (format #f "~aport expects: -\n" (location-prefix ast))))
                (map
                 (lambda (ast)
                   (if ast
-                      (format #f "~aport accept: ~a\n" (step->location ast) (trigger->string ast))
+                      (format #f "~aport expects: ~a\n" (location-prefix ast) (trigger->string ast))
                       (let* ((ast ((compose .statement .behavior .type .ast .port) status))
-                             (location (step->location ast)))
-                        (format #f "~aport accept: ~a\n"  location "-"))))
+                             (location (location-prefix ast)))
+                        (format #f "~aport expects: ~a\n"  location "-"))))
                 port-acceptances)))
           "")))
       (($ <fork-error>)
        (let* ((action (.ast status))
-              (location (step->location action))
+              (location (location-prefix action))
               (action-port (.port action))
               (action-port-name (.name action-port))
               (first (.action status)))
@@ -844,58 +847,61 @@ intermediate steps such as assignments, function calls, replies,
           (list
            (format #f "~aforking not allowed, action on port: ~a\n" location action-port-name)
            (if first
-               (let* ((first-location (step->location first))
+               (let* ((first-location (location-prefix first))
                       (first-port-name (.name (.port first))))
                  (format #f "~aafter action on port: ~a\n" first-location first-port-name))
                (let* ((trigger (.trigger pc))
-                      (trigger-location (step->location (parent action <on>)))
+                      (trigger-location (location-prefix (parent action <on>)))
                       (trigger-port (.port trigger))
                       (trigger-port-name (.name trigger-port)))
                  (format #f "~afor trigger on port ~a\n" trigger-location trigger-port-name))))
           "")))
       (($ <refusals-error>)
-       (let* ((refusals (sort (.refusals status) equal?))
-              (refusals (apply append refusals))
-              (location (or (and=> (.ast status) step->location)
-                            "<unknown-file>:"))
-              (inevitables optionals
-                           (partition
-                            (compose (cute equal? "inevitable" <>) car)
-                            refusals)))
-         (string-append
-          (format #f "~acomponent can omit all of: ~a\n"
-                  location
-                  (string-join (map cadr refusals) ", "))
-          (format #f "~athe interface promises inevitably: ~a \n"
-                  location
-                  (string-join (map cadr inevitables) ","))
-          (if (null? optionals) ""
-              (format #f "~aor optionally: ~a\n"
-                      location
-                      (string-join (map cadr optionals) ","))))))
+       (match (.refusals status)
+         ((and ($ <trigger>) trigger)
+          (let ((location (location-prefix trigger))
+                (trigger (trigger->string trigger)))
+            (format #f "~acomponent can omit return of trigger: ~a\n"
+                    location trigger)))
+         (_
+          (let* ((refusals (sort (.refusals status) equal?))
+                 (refusals (apply append refusals))
+                 (location (and=> (.ast status) location-prefix))
+                 (inevitables optionals
+                              (partition
+                               (compose (cute equal? "inevitable" <>) car)
+                               refusals)))
+            (string-append
+             (format #f "~acomponent can omit all of: ~a\n"
+                     location
+                     (string-join (map cadr refusals) ", "))
+             (format #f "~athe interface promises inevitably: ~a \n"
+                     location
+                     (string-join (map cadr inevitables) ","))
+             (if (null? optionals) ""
+                 (format #f "~aor optionally: ~a\n"
+                         location
+                         (string-join (map cadr optionals) ","))))))))
       (($ <range-error>)
        (let* ((variable (.variable status))
               (name (.name variable))
               (type (.type variable))
               (value (.value status))
-              (location (or (step->location (.ast status))
-                            "<unknown-file>:")))
+              (location (location-prefix (.ast status))))
          (string-join
           (list
-           (format #f "~ainfo: ~a `~a' defined here\n" (step->location variable) (ast-name variable) name)
-           (format #f "~ainfo: of type `~a' defined here\n" (step->location type) (ast:name type)))
+           (format #f "~ainfo: ~a `~a' defined here\n" (location-prefix variable) (ast-name variable) name)
+           (format #f "~ainfo: of type `~a' defined here\n" (location-prefix type) (ast:name type)))
           "")))
       (($ <second-reply-error>)
        (let* ((previous (.previous status))
-              (location (or (step->location (.ast status))
-                            "<unknown-file>:")))
-         (format #f "~ainfo: reply previously set here\n" (step->location previous))))
+              (location (location-prefix (.ast status))))
+         (format #f "~ainfo: reply previously set here\n" (location-prefix previous))))
       ((and (? (is? <match-error>)) (= .ast #f))
-       (let ((location "<unknown-file>:"))
+       (let ((location (location-prefix #f)))
          (format #f "~aerror: no match; got input `~s'\n" location (.input status))))
       ((and (? (is? <match-error>)) (= .ast ast))
-       (let ((location (or (step->location ast)
-                           "<unknown-file>:"))
+       (let ((location (location-prefix ast))
              (label (label->string ast)))
          (if (not label)
              (format #f "~aerror: no match; got input `~s'\n"
@@ -909,8 +915,7 @@ intermediate steps such as assignments, function calls, replies,
                                   (.port.name ast)))
                    (instance (if instance (runtime:instance->string instance)
                                  port-name))
-                   (location (or (step->location (.ast status))
-                                 "<unknown-file>:"))
+                   (location (location-prefix (.ast status)))
                    (prefix (if (is-a? (%sut) <runtime:port>) ""
                                (string-append instance ".")))
                    (ast (trigger->string (clone ast #:port.name #f))))
@@ -923,8 +928,7 @@ intermediate steps such as assignments, function calls, replies,
          (ast (.ast status))
          (port-name (if instance (string-join (runtime:instance->path instance) ".")
                         (.port.name ast)))
-         (location (or (step->location (.ast status))
-                       "<unknown-file>:"))
+         (location (location-prefix (.ast status)))
          (ast (trigger->string (clone ast #:port.name #f)))
          (labels (ast:label* status))
          (labels (map label->string labels))
@@ -935,7 +939,23 @@ intermediate steps such as assignments, function calls, replies,
 (define* (report traces #:key eligible (trace "event") internal? locations? state? verbose?)
   "Print any error messages and the trail of (car TRACES) in
 trace-format TRACE, given that the trail is the same for all TRACES."
-  (let* ((trace-format trace)
+
+  (define (trace< a b)
+    "Compare traces A and B on trail length, trace length or string
+value of the trail."
+    (define (length< a b)
+      (< (length a) (length b)))
+    (let* ((trail-a (trace->string-trail a))
+           (trail-b (trace->string-trail b))
+           (result (length< trail-a trail-b)))
+      (if (or result (length< trail-b trail-a)) result
+          (let ((result (length< trail-a trail-b)))
+            (if (or result (length< trail-b trail-a)) result
+                (string< (string-join trail-a)
+                         (string-join trail-b)))))))
+
+  (let* ((traces (sort traces trace<))
+         (trace-format trace)
          (trace (and (pair? traces) (car traces)))
          (pc (and trace (car trace)))
          (status (and pc (.status pc)))
@@ -983,7 +1003,7 @@ trace-format TRACE, given that the trail is the same for all TRACES."
                eligible)
       (let* ((eligible (if (and (is-a? status <end-of-trail>) (.ast status)) (end-of-trail-labels pc)
                            eligible))
-             (labels (labels))
+             (labels (labels pc))
              (labels (if (and (pair? eligible) (not (member (car eligible) labels))) eligible
                          labels)))
         (format #t "~s\n" (cons 'labels labels))
