@@ -1,7 +1,7 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2019, 2020, 2021 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
-;;; Copyright © 2020, 2021 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2020, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2021 Paul Hoogendijk <paul@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
@@ -41,8 +41,8 @@
   #:use-module (dzn vm step)
   #:use-module (dzn vm util)
   #:export (%exploring?
-            %strict?
             did-provides-out?
+            filter-compliance-error
             filter-error
             filter-illegal+implicit-illegal
             filter-implicit-illegal
@@ -57,10 +57,12 @@
             run-async-event
             run-external
             run-external-q
+            run-external-modeling
             run-requires
             run-silent
             run-to-completion
-            run-to-completion*))
+            run-to-completion*
+            run-to-completion*-context-switch))
 
 ;;; Commentary:
 ;;;
@@ -69,9 +71,6 @@
 ;;;
 ;;; Code:
 
-;; Is the input trail to be matched exactly?
-(define %strict? (make-parameter #f))
-
 ;; Are we running "explore"?
 (define %exploring? (make-parameter #f))
 
@@ -86,6 +85,15 @@
 (define (trace-valid? trace)
   ((compose not .status car) trace))
 
+(define (filter-compliance-error traces)
+  (let ((non-compliance rest
+                        (partition
+                         (disjoin (compose (is-status? <compliance-error>) car)
+                                  (compose (is-status? <match-error>) car))
+                         traces)))
+    (if (pair? rest) rest
+        non-compliance)))
+
 (define (filter-error traces)
   (let ((error rest (partition
                      (compose (conjoin (is? <error>)
@@ -109,10 +117,14 @@
         illegal)))
 
 (define (filter-implicit-illegal-only traces)
-  (let ((illegal rest (partition
-                       (conjoin (compose (is-status? <implicit-illegal-error>) car)
-                                (compose (cute equal? <> 2) length))
-                       traces)))
+  (let ((illegal
+         rest (partition
+               (conjoin (compose (is-status? <implicit-illegal-error>) car)
+                        (compose (match-lambda
+                                   ((trigger "<illegal>") #t)
+                                   (_ #f))
+                                 trace->string-trail))
+               traces)))
     (if (pair? rest) rest
         illegal)))
 
@@ -153,6 +165,68 @@ valid PCs that are executing an imperative statement."
                                    pcs)))
        (> (length imperative) 1)))))
 
+(define-method (block-on-port trace)
+  "Return TRACE blocked on port when trace head is an <action> or a
+<trigger-return> on a blocking requires port, unless already blocked on
+this statement.  Otherwise, return #f.
+
+Blocking and resettiing the STATEMENT on the TRACE makes RTC?  true,
+which creates an opening to accept new modeling events.  Continuation of
+the <action> or <trigger-return> happens when revisiting this method
+with a TRACE that has BLOCKED still set to this port."
+
+  (define (modeling-traces pc instance)
+    (let* ((port (.ast instance))
+           (ipc (clone pc #:trigger #f #:previous #f #:instance
+                       #f #:trail '() #:statement #f))
+           (modeling (modeling-names (.type port)))
+           (traces (parameterize ((%sut instance)
+                                  (%liveness? 'port)
+                                  (%exploring? #f)
+                                  (%strict? #f))
+                     (append-map (cut run-to-completion ipc <>) modeling)))
+           (traces (filter (compose (negate (is-status? <error>)) car) traces)))
+      traces))
+
+  (define (requires-modeling-armed? pc)
+    (let* ((instance (.instance pc))
+           (requires-ports (filter
+                            (conjoin runtime:boundary-port?
+                                     ast:requires?
+                                     (negate (cute eq? <> instance)))
+                            (%instances)))
+           (modeling (filter (compose pair? modeling-names .type .ast)
+                             requires-ports))
+           (ports (filter (compose pair?
+                                   (cute modeling-traces pc <>))
+                          modeling)))
+      (pair? ports)))
+
+  (let* ((pc (car trace))
+         (instance (.instance pc))
+         (statement (.statement pc))
+         (blocked (.blocked pc)))
+    (and
+     (not (is-a? (%sut) <runtime:port>))
+     (is-a? instance <runtime:port>)
+     (ast:requires? instance)
+     (ast:blocking? instance)
+     (not (ast:modeling? (.trigger pc)))
+     (or (is-a? statement <action>)
+         (is-a? statement <trigger-return>))
+     (let ((blocked-port (and (pair? blocked) (caar blocked)))
+           (blocked-pc (and (pair? blocked) (cdar blocked))))
+       (and
+        (not (ast:eq? statement (and=>  blocked-pc .statement)))
+        (or (and (is-a? (%sut) <runtime:system>)
+                 (> (length (ast:provides-port* (runtime:%sut-model))) 1))
+            (requires-modeling-armed? pc))
+        (let* ((trace (cdr trace))
+               (pc (blocked-on-boundary-reset pc))
+               (pc (block pc instance))
+               (trace (cons pc trace)))
+          trace))))))
+
 (define (mark-determinism-error trace)
   "Truncate TRACE up to including the component <initial-compound> and
 mark it with <determinism-error>."
@@ -180,7 +254,8 @@ mark it with <determinism-error>."
                                 (cute pc-equal? <> pc-loop)
                                 (reverse trace-prefix))
                                (cute - (length trace-prefix) <> 1)))
-         (rest shorter-prefix (split-at trace-prefix shorter-index))
+         (shorter-prefix (if (not shorter-index) trace-suffix
+                             (drop trace-prefix shorter-index)))
          (pc (car shorter-prefix))
          (ast (or (.statement pc) ast-model))
          (error (make <livelock-error> #:ast ast #:message "livelock"))
@@ -197,10 +272,11 @@ mark it with <determinism-error>."
 prefix."
   (define* (trace-head-recurrence? trace)
     (let ((trace (filter
-                  (conjoin (compose (is? <runtime:component>) .instance)
-                           (negate (compose (is? <initial-compound>) .statement))
-                           (negate (compose (is? <end-of-on>) .statement))
-                           )
+                  (conjoin
+                   (disjoin (compose (cute eq? <> (%sut)) .instance)
+                            (compose (is? <runtime:component>) .instance))
+                   (negate (compose (is? <initial-compound>) .statement))
+                   (negate (compose (is? <end-of-on>) .statement)))
                   trace)))
       (and (pair? trace)
            (find (cute pc-equal?
@@ -214,6 +290,9 @@ prefix."
             (%livelock-threshold (* 2 (%livelock-threshold)))
             #f)
            ((pc tail ...)
+            (%debug "  ~s ~s <livelock>\n"
+                    ((compose name .instance) pc)
+                    (and=> (.trigger pc) trigger->string))
             (let ((index (and=> (list-index (cute pc-equal? <> pc)
                                             (reverse trace))
                                 (cute - (length trace) <> 1))))
@@ -226,16 +305,30 @@ prefix."
   "Return a list of traces, produced by appending TRACE to each of the
 program-counters produced by taking a step."
 
-  (define* ((mark-pc input o) pc)
-    (%debug "match fail, ast ~s ~a, input ~s\n" (name o)
-            (and (is-a? o <action>) (trigger->string o))
-            input)
-    (cond ((.status pc)
-           pc)
-          ((and (%strict?) input)
-           (clone pc #:status (make <match-error> #:ast o #:input input #:message "match")))
-          (else
-           (clone pc #:status (make <postponed-match> #:ast o #:input input)))))
+  (define* ((mark-pc input orig-pc) pc)
+    (let ((statement (.statement orig-pc)))
+      (%debug "match fail, ast ~s ~a, input ~s\n" (name statement)
+              (and (is-a? statement  <action>)
+                   (and=> (trace->trail orig-pc) cdr))
+              input)
+     (cond ((.status pc)
+            pc)
+           ((or (and (%strict?) input)
+                (and input
+                     (is-a? statement <trigger-return>)
+                     (blocked-on-boundary? orig-pc)))
+            (let ((error (make <match-error> #:ast statement #:input input
+                               #:message "match")))
+              (clone pc #:status error)))
+           (else
+            (clone pc #:status (make <postponed-match> #:ast statement
+                                     #:input input))))))
+
+  (define (blocked-collaterally? orig-pc pc)
+    (match (.collateral pc)
+      (((port . pc) t ...)
+       (eq? pc orig-pc))
+      (_ #f)))
 
   (let loop ((trace trace))
     (let ((pc (car trace)))
@@ -244,7 +337,10 @@ program-counters produced by taking a step."
             ((livelock? trace)
              =>
              (compose list (cute mark-livelock-error trace <>)))
-            ((rtc? pc) (list trace))
+            ((rtc? pc)
+             (list trace))
+            ((block-on-port trace)
+             => list)
             (else
              (let* ((o (.statement pc))
                     (observable? (or (is-a? o <action>)
@@ -252,7 +348,13 @@ program-counters produced by taking a step."
                                      (is-a? o <trigger-return>)))
                     (observable (and observable? (and=> (trace->trail pc) cdr)))
                     (pcs (step pc o))
-                    (input pc (if (and observable (not (interactive?))) ((%next-input) pc) (values #f pc)))
+                    (new-pc (and (pair? pcs) (car pcs)))
+                    (trace (if (any (cute blocked-collaterally? pc <>) pcs)
+                               (cdr trace)
+                               trace))
+                    (input pc (if (and observable (not (interactive?)))
+                                  ((%next-input) pc)
+                                  (values #f pc)))
                     (pcs (cond ((%exploring?)
                                 pcs)
                                ((not observable)
@@ -260,13 +362,14 @@ program-counters produced by taking a step."
                                ((equal? input observable)
                                 (map (cute clone <> #:trail (.trail pc)) pcs))
                                (else
-                                (map (mark-pc input o) pcs))))
+                                (map (mark-pc input pc) pcs))))
                     (traces (map (cut cons <> trace) pcs)))
                (cond
                 ((ast:declarative? o)
-                 (let ((declarative imperative
-                                    (partition (compose ast:declarative? .statement car)
-                                               traces)))
+                 (let ((declarative
+                        imperative (partition
+                                    (compose ast:declarative? .statement car)
+                                    traces)))
                    (cond
                     ((pair? declarative)
                      (append imperative (append-map loop declarative)))
@@ -342,10 +445,20 @@ PC until RTC?."
       (map (cute rewrite-trace-head (cute set-end-of-trail labels <>) <>)
            traces)))
 
+  (define (previous-provides? trace)
+    (let ((previous (any .trigger trace)))
+      (and previous
+           (and=> (.port previous) ast:provides?))))
+
+  (define (must-switch? trace)
+    (and (not (is-a? (%sut) <runtime:port>))
+         (not (previous-provides? trace))))
+
   (define (loop traces)
     (let* ((traces (if (%exploring?) traces (filter-illegal traces)))
            (traces (filter-match-error traces))
            (traces (filter-postponed-match traces))
+           (traces (filter-implicit-illegal traces))
            (pcs (map car traces)))
       (cond
        ((null? traces)
@@ -356,7 +469,11 @@ PC until RTC?."
        ((every (conjoin (negate (is-status? <postponed-match>))
                         (disjoin rtc? (is-status? <match-error>)))
                pcs)
-        (filter-implicit-illegal traces))
+        (let* ((to-switch traces (partition must-switch? traces))
+               (traces (append traces (map switch-context to-switch)))
+               (done todo (partition rtc? traces)))
+          (append done
+                  (loop (append-map extend-trace todo)))))
        ((postponed-match? traces)
         =>
         (lambda (traces)
@@ -383,7 +500,8 @@ PC until RTC?."
 (define-method (run-to-completion-unmemoized (pc <program-counter>) event)
   "Return a list of traces produced by taking steps, starting from PC
 with EVENT as first step, until RTC?."
-  (let ((pc (begin-step pc event)))
+  (let ((pc (if (eq? event 'rtc) pc
+                (begin-step pc event))))
     (run-to-completion-unmemoized pc)))
 
 (define run-to-completion
@@ -392,7 +510,9 @@ with EVENT as first step, until RTC?."
     (lambda (pc event)
       "Memoizing version of RUN-TO-COMPLETION-UNMEMOIZED."
       (if (not (%exploring?)) (run-to-completion-unmemoized pc event)
-          (let* ((event-string (if (string? event) event (trigger->string event)))
+          (let* ((event-string (cond ((string? event) event)
+                                     ((eq? event 'rtc) "*rtc*")
+                                     (else (trigger->string event))))
                  (sut (%sut))
                  (root (parent (.ast sut) <root>))
                  (key (string-append
@@ -441,21 +561,16 @@ until RTC?."
 
 (define-method (run-silent (pc <program-counter>) (port <runtime:port>))
   (define (update-state pc port-pc)
-    (let ((pc (set-state pc (get-state port-pc port))))
-      (clone pc #:external-q (.external-q port-pc))))
+    (set-state pc (get-state port-pc port)))
   (%debug "run-silent... ~s\n" (name port))
   (let ((modeling-names (modeling-names port)))
     (if (null? modeling-names) '()
-        (let* ((previous (.previous pc))
-               (ipc (clone pc #:trigger #f #:previous #f #:instance #f #:trail '() #:statement #f))
-               (r:other-port (runtime:other-port port))
-               (external? (ast:external? (.ast r:other-port)))
+        (let* ((ipc (clone pc #:trigger #f #:previous #f #:instance #f #:trail '() #:statement #f))
                (traces (parameterize ((%sut port)
                                       (%exploring? #t)
-                                      (%strict? (not external?)))
+                                      (%strict? #t))
                          (append-map (cut run-to-completion ipc <>) modeling-names)))
-               (traces (filter (conjoin (disjoin (const external?)
-                                                 (compose null? trace->trail))
+               (traces (filter (conjoin (compose null? trace->trail)
                                         (compose (negate .status) car))
                                traces)))
           (map (cute rewrite-trace-head (cute update-state pc <>) <>) traces)))))
@@ -468,6 +583,50 @@ until RTC?."
          (traces (run-silent pc port-instance)))
     (map car traces)))
 
+(define-method (run-silent traces)
+  (let* ((pcs (map last traces))
+         (pcs (map (cute clone <> #:status #f) pcs)))
+    (append-map (cute run-silent <> (%sut)) pcs)))
+
+(define-method (run-external-modeling (pc <program-counter>) (port <runtime:port>))
+  (define (update-state pc port-pc)
+    (let ((pc (set-state pc (get-state port-pc port))))
+      (clone pc #:external-q (.external-q port-pc) #:status (.status port-pc))))
+  (%debug "run-external-modeling... ~s\n" (name port))
+  (let* ((r:other-port (runtime:other-port port))
+         (external? (and (ast:requires? r:other-port)
+                         (ast:external? r:other-port))))
+    (if (not external?) '()
+        (let ((modeling-names (modeling-names port)))
+          (if (null? modeling-names) '()
+              (let* ((previous (.previous pc))
+                     (ipc (clone pc #:trigger #f #:previous #f #:instance #f
+                                 #:trail '() #:statement #f))
+                     (traces (parameterize ((%sut port)
+                                            (%liveness? 'component)
+                                            (%exploring? #t)
+                                            (%strict? #f))
+                               (append-map (cut run-to-completion ipc <>) modeling-names)))
+                     (traces (filter (compose
+                                      (disjoin not (is? <queue-full-error>))
+                                      .status car)
+                                     traces)))
+                (map (cute rewrite-trace-head (cute update-state pc <>) <>) traces)))))))
+
+(define-method (run-external-modeling (pc <program-counter>))
+  (let* ((ports (filter (conjoin runtime:boundary-port?
+                                 ast:external? ast:requires?)
+                        (%instances))))
+    (append-map (cute run-external-modeling pc <>) ports)))
+
+(define-method (run-external-modeling (pc <program-counter>) event)
+  (let* ((component ((compose .type .ast) (%sut)))
+         (trigger (clone (string->trigger event) #:parent component))
+         (port-name (.port.name trigger))
+         (port-instance (runtime:port-name->instance port-name))
+         (traces (run-external-modeling pc port-instance)))
+    (map car traces)))
+
 (define-method (run-interface (pc <program-counter>) (event <string>))
   (define (event-executed? trace)
     (let ((trail (trace->string-trail trace)))
@@ -495,8 +654,10 @@ until RTC?."
 
 (define-method (run-requires (pc <program-counter>) event)
   (define (event-executed? port-instance trace)
-    (let ((trail (map cdr (trace->trail trace))))
-      (or (and (pair? trail)
+    (let ((status (.status (car trace)))
+          (trail (map cdr (trace->trail trace))))
+      (or (is-a? status <queue-full-error>)
+          (and (pair? trail)
                (equal? event (car trail)))
           (and (null? trail)
                (let* ((pc (car trace))
@@ -515,15 +676,21 @@ until RTC?."
          (modeling-events (map (cut string-append port-name "." <>) modeling-names))
          (traces (append-map (cut run-to-completion pc <>) modeling-events))
          (traces (filter (cute event-executed? port-instance <>) traces))
-         (illegals (filter (compose (is-status? <illegal-error>) car) traces))
-         (traces (filter (compose (negate .status) car) traces)))
+         (errors (filter (compose .status car) traces)))
 
-    (if (pair? illegals) illegals
+    (if (pair? errors) errors
         (let* ((component-port (runtime:other-port port-instance))
-               (component-trigger (trigger->component-trigger component-port trigger))
+               (component-trigger (trigger->component-trigger component-port
+                                                              trigger))
                (instance (.container component-port))
-               (traces (map (cut rewrite-trace-head (cut clone <> #:instance instance) <>) traces))
-               (traces (append-map run-flush traces)))
+               (traces (map (cut rewrite-trace-head
+                              (cut clone <> #:instance instance) <>)
+                            traces))
+               (collateral-blocked? (or (blocked-on-boundary? pc)
+                                        (and (get-handling pc instance)
+                                             (blocked-port pc instance))))
+               (traces (if collateral-blocked? traces
+                           (append-map run-flush traces))))
           traces))))
 
 (define-method (run-async-event (pc <program-counter>))
@@ -601,30 +768,77 @@ until RTC?."
                (let* ((model (runtime:%sut-model))
                       (ast (if (is-a? model <system>) model
                                (.behavior model)))
-                      (pc (clone pc #:status (make <match-error> #:ast ast #:input event #:message "match"))))
+                      (error (make <match-error> #:ast ast #:input event
+                                   #:message "match"))
+                      (pc (clone pc #:status error)))
+                 (%debug "<match> ~a pc: ~s\n" event pc)
                  (list (list pc) )))))))
 
 (define-method (run-to-completion* (pc <program-counter>) event)
-  (let ((pc (clone pc #:instance #f)))
-    (cond
-     ((is-a? (%sut) <runtime:port>)
-      (run-interface pc event))
-     ((external-trigger-in-q? pc event)
-      (run-external pc event))
-     ((external-trigger? event)
-      (let ((pcs (cons pc (run-silent pc event))))
-        (append-map (cute run-external <> event) pcs)))
-     ((requires-trigger? event)
-      (let ((async-traces (if (null? (.async pc)) '()
-                              (flush-async-event pc event))))
-        (if (pair? async-traces) async-traces
-            (let ((pcs (cons pc (run-silent pc event))))
-              (append-map (cute run-requires <> event) pcs)))))
-     ((provides-trigger? event)
-      (run-to-completion pc event))
-     ((async-event? pc event)
-      (flush-async-event pc event))
-     ((and (eq? event #f) (pair? (.async pc)))
-      (flush-async pc))
-     (else
-      '()))))
+  (define (illegal-trace pc)
+    (let ((illegal (make <implicit-illegal-error> #:ast (.trigger pc)
+                         #:message "illegal")))
+      (list (list (clone pc #:status illegal)))))
+  (define (modeling? trace)
+    (and=> (any .trigger (reverse trace)) ast:modeling?))
+  (%debug "run-to-completion*: ~a\n" event)
+  (cond
+   ((is-a? (%sut) <runtime:port>)
+    (run-interface pc event))
+   ((external-trigger-in-q? pc event)
+    (run-external pc event))
+   ((external-trigger? event)
+    (let ((pcs (cons pc (run-external-modeling pc event))))
+      (append-map (cute run-external <> event) pcs)))
+   ((requires-trigger? event)
+    (let ((async-traces (if (null? (.async pc)) '()
+                            (flush-async-event pc event))))
+      (if (pair? async-traces) async-traces
+          (let* ((pcs (cons pc (run-external-modeling pc event)))
+                 (port (.port (string->trigger event)))
+                 (blocked-port (and=> (blocked-on-boundary? pc event) .ast))
+                 (traces (append-map (cute run-requires <> event) pcs)))
+            (if (or (not port) (not (ast:eq? port blocked-port))) traces
+                (filter (negate modeling?) traces))))))
+   ((provides-trigger? event)
+    (if (blocked-on-boundary-provides? pc event) (illegal-trace pc)
+        (run-to-completion pc event)))
+   ((async-event? pc event)
+    (flush-async-event pc event))
+   ((and (eq? event #f) (pair? (.async pc)))
+    (flush-async pc))
+   (else
+    '())))
+
+(define-method (run-to-completion* trace event)
+  (extend-trace trace (cute run-to-completion* <> event)))
+
+(define-method (run-to-completion*-context-switch (pc <program-counter>) event)
+  (%debug "run-to-completion*-switch-context: ~a\n" event)
+  (let* ((orig-pc pc)
+         (blocked-on-action? (blocked-on-action? pc event))
+         (pc (if (and (blocked-on-boundary? pc event)
+                      (or blocked-on-action? (return-trigger? event)))
+                 (blocked-on-boundary-switch-context pc event)
+                 pc))
+         (switched? (not (eq? pc orig-pc)))
+         (trigger? (or (is-a? (%sut) <runtime:port>)
+                       (provides-trigger? event)
+                       (and (not blocked-on-action?)
+                            (requires-trigger? event))
+                       (async-event? pc event)))
+         (pc (if (or switched? trigger?) pc
+                 (blocked-on-boundary-collateral-release pc)))
+         (pc (if (provides-trigger? event) pc
+                 (switch-context pc)))
+         (switched? (not (eq? pc orig-pc)))
+         (pc (if switched? pc
+                 (clone pc #:instance #f)))
+         (skip-rtc? (or (not switched?)
+                        (provides-trigger? event)))
+         (pc (if (or trigger? (not switched?) (rtc-event? event)) pc
+                 (clone pc #:trail (cons event (.trail pc)))))
+         (traces (if skip-rtc? (run-to-completion* pc event)
+                     (run-to-completion pc 'rtc))))
+    (if (or skip-rtc? (not trigger?)) traces
+        (append-map (cute run-to-completion* <> event) traces))))
