@@ -1,9 +1,9 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2017, 2019, 2020 Rob Wieringa <rma.wieringa@gmail.com>
-;;; Copyright © 2014, 2017, 2020 Rutger van Beusekom <rutger@dezyne.org>
-;;; Copyright © 2020, 2021 Paul Hoogendijk <paul@dezyne.org>
+;;; Copyright © 2014, 2017, 2020, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2020, 2021, 2022 Paul Hoogendijk <paul@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -85,12 +85,24 @@
    (if (.behavior o) (wfc (.behavior o))
        `(,(wfc-error o "interface must define a behavior")))))
 
-(define-method (wfc (o <component-model>)) ;; is-a <model>
-  (append-map wfc (ast:port* o)))
+(define-method (wfc (o <foreign>))
+  (append
+   (re-definition o)
+   (let* ((root (parent o <root>))
+          (basename (ast:base-name root)))
+     (if (not (equal? (string-join (ast:full-name o) "_") basename)) '()
+         `(,(wfc-error
+             o
+             (format
+              #f
+              "foreign component cannot have the same name as its file: `~a'"
+              basename)))))
+   (blocking-ports o)))
 
 (define-method (wfc (o <component>)) ;; is-a <component-model>
   (append
    (re-definition o)
+   (blocking-ports o)
    (if (> (length (ast:provides-port* o)) 0) '()
        `(,(wfc-error o "component with behavior must define a provides port")))
    (let ((port-errors (append-map wfc (ast:port* o)))
@@ -161,7 +173,7 @@
             `(,(wfc-error o (format #f "undefined interface `~a'" (type-name (.type.name o))))))
            ((not (is-a? interface <interface>))
             `(,(wfc-error o (format #f "interface expected, found: `~a ~a'" (ast-name interface) (type-name interface)))))
-           ((and (.injected o)
+           ((and (.injected? o)
                  (let ((out-events (filter ast:out? (ast:event* o))))
                    (and (pair? out-events)
                         `(,(wfc-error o (format #f "injected port `~a' has out events: ~a" (.name o)
@@ -181,6 +193,16 @@
    (wfc (.signature o))))
 
 (define-method (wfc (o <signature>))
+  (define (check-formal function formal)
+    (let ((message
+           (format
+            #f
+            "type `~a' cannot be used for `~a' parameter `~a' in function `~a'"
+            (ast:name (.type.name formal))
+            (.direction formal)
+            (.name formal)
+            (.name function))))
+      (wfc-error formal message)))
   (append
    (let ((type (ast:type o)))
      (cond ((not type)
@@ -188,13 +210,19 @@
            ((is-a? type <extern>)
             `(,(wfc-error o (format #f "cannot use extern type `~a' as return type" (type-name (.type.name o))))))
            (else '())))
-   (append-map wfc (ast:formal* o))))
+   (append-map wfc (ast:formal* o))
+   (let ((function (parent o <function>)))
+     (if (not function) '()
+         (let ((formals (filter (conjoin (disjoin ast:out? ast:inout?)
+                                         (negate (compose (is? <extern>) .type)))
+                                (ast:formal* o))))
+           (map (cute check-formal function <>) formals))))))
 
 (define ((argument-type-check o) argument formal)
   (let ((argument-type (ast:type argument))
         (formal-type (ast:type formal)))
-    (cond ((not argument-type)
-           `(,(wfc-error o (format #f "undefined identifier `~a'" (.name argument)))))
+    (cond ((or (not formal-type) (not argument-type))
+           '())
           ((equal-type? argument-type formal-type)
            '())
           (else
@@ -392,8 +420,6 @@
             ((parent (.parent o) <blocking>)
              `(,(wfc-error o "nested blocking used")
                ,(wfc-info (parent (.parent o) <blocking>) "within blocking here")))
-            ((> (length (ast:provides-port* model)) 1)
-             `(,(wfc-error o "cannot use blocking with multiple provide ports")))
             (else '()))))
   (append (blocking o) (wfc (.statement o))))
 
@@ -403,9 +429,7 @@
      (let ((parent (parent (.parent o) <on>)))
        (if parent `(,(wfc-error o "nested on used")
                     ,(wfc-info parent "within on here"))
-           '()))
-     (if (is-a? (parent o <model>) <interface>) (modeling-silent o)
-         '())))
+           '()))))
   (append
    (on o)
    (append-map wfc (ast:trigger* o))
@@ -630,6 +654,19 @@
            `(,(wfc-error o (format #f "~a expression expected" (class-name type)))))
           (else '()))))
 
+(define-method (no-extern-expression (o <expression>))
+  (let* ((expr-wfc (wfc o))
+         (expr-type (ast:type o)))
+    (cond ((pair? expr-wfc) expr-wfc)
+          ((not expr-type)
+           `(,(wfc-error o (format #f "undefined identifier `~a'" (.name o)))))
+          ((is-a? expr-type <extern>)
+           `(,(wfc-error
+               o
+               (format #f "extern data-type `~a' expression in binary operator"
+                       (type-name expr-type)))))
+          (else '()))))
+
 (define-method (wfc (o <not>))
   (typed-expression (.expression o) <bool>))
 
@@ -640,12 +677,12 @@
          (expr2-wfc (typed-expression expr2 type)))
     (append expr1-wfc expr2-wfc)))
 
-(define-method (binary-equal-type (o <binary>))
+(define-method (binary-equal-no-extern-type (o <binary>))
   (let* ((expr1 (.left o))
-         (expr1-wfc (wfc  expr1))
+         (expr1-wfc (no-extern-expression expr1))
          (expr1-type (ast:type expr1))
          (expr2 (.right o))
-         (expr2-wfc (wfc expr2))
+         (expr2-wfc (no-extern-expression expr2))
          (expr2-type (ast:type expr2)))
     (cond ((or (pair? expr1-wfc) (pair? expr2-wfc)) (append expr1-wfc expr2-wfc))
           ((not (equal-type? expr1-type expr2-type))
@@ -657,8 +694,8 @@
 (define-method (wfc (o <and>)) (typed-binary o <bool>))
 (define-method (wfc (o <or>)) (typed-binary o <bool>))
 
-(define-method (wfc (o <equal>)) (binary-equal-type o))
-(define-method (wfc (o <not-equal>)) (binary-equal-type o))
+(define-method (wfc (o <equal>)) (binary-equal-no-extern-type o))
+(define-method (wfc (o <not-equal>)) (binary-equal-no-extern-type o))
 
 (define-method (wfc (o <greater-equal>)) (typed-binary o <int>))
 (define-method (wfc (o <greater>)) (typed-binary o <int>))
@@ -724,11 +761,16 @@
 (define-method (tail-recursion (o <call>))
   (let ((function (parent o <function>)))
     (if (or (not function) (not (.recursive function))) '()
-            (let* ((continuation ((compose car wfc:continuation) o))
-                   (continuation (if (or (parent o <assign>) (parent o <variable>)) ((compose car (@@ (dzn code makreel) makreel:continuation)) continuation) continuation))
-                   (continuation (and continuation
-                                      (not (ast:eq? continuation (.statement (parent o <function>))))
-                                      continuation)))
+        (let* ((continuation ((compose car wfc:continuation) o))
+               (continuation (if (or (parent o <assign>)
+                                     (parent o <reply>)
+                                     (parent o <return>)
+                                     (parent o <variable>))
+                                 ((compose car (@@ (dzn code makreel) makreel:continuation)) continuation)
+                                 continuation))
+               (continuation (and continuation
+                                  (not (ast:eq? continuation (.statement (parent o <function>))))
+                                  continuation)))
               (cond ((is-a? continuation <return>)
                      `(,(wfc-error o "cannot use valued function in recursion")
                        ,(wfc-info continuation "statement after call")))
@@ -749,14 +791,6 @@
                  (list (wfc-error ast "imperative statement expected"))))
           '())))
 
-(define-method (modeling-silent (o <on>))
-  (if (and (or (eq? (.silent? o) *unspecified*)
-               (is-a? (.silent? o) <ast>)) ((@@ (dzn parse silence) any-modeling?) o))
-      `(,(wfc-error o (format #f "cannot determine silence"))
-        ,@(if (is-a? (.silent? o) <ast>) `(,(wfc-error (.silent? o) (format #f "may communicate or be silent")))
-              '()))
-      '()))
-
 (define-method (re-definition (o <declaration>))
   (let* ((name (ast:name o))
          (scope (decl-scope o))
@@ -843,7 +877,8 @@
          (event (and (not unblock?) (.event trigger)))
          (event-type (and event (ast:type event)))
          (reply-type (ast:type o)))
-    (cond ((and (not (%model-blocking?)) port (not (equal? (.port.name o) (.port.name trigger))))
+    (cond ((and (not (%model-blocking?)) port (ast:provides? trigger)
+                (not (equal? (.port.name o) (.port.name trigger))))
            `(,(wfc-error o (format #f "port `~a' does not match with trigger port `~a'"
                                    (.port.name o) (.port.name trigger)))))
           ((and (not unblock?) (not event)) '()) ; already covered in trigger check
@@ -855,9 +890,6 @@
                                        (type-name reply-type)))
                  ,(wfc-info event "event defined here"))
                '()))
-          ((not (%model-blocking?))     ; also covers interfaces
-           `(,(wfc-error o (format #f "cannot use reply on requires out-trigger `~a'" (.name event)))
-             ,(wfc-info event (format #f "event `~a' defined here" (.name event)))))
           (else (wfc-reply-expression o port)))))
 
 (define-method (action (o <action>))
@@ -919,8 +951,9 @@
   (append-map binding-direction (ast:binding* o)))
 
 (define-method (binding-direction (o <binding>))
-  (let ((left (.left o))
-        (right (.right o)))
+  (let* ((o (ast:normalize o))
+         (left (.left o))
+         (right (.right o)))
     (cond ((and (ast:wildcard? (.port.name left))
                 (ast:wildcard? (.port.name right)))
            `(,(wfc-error o "cannot bind two wildcards")))
@@ -953,24 +986,33 @@
              ,(wfc-info (.port right) (format #f "port `~a' defined here" (.port.name right)))))
           ((and
             (.port left)
-            (ast:requires? (.port left))
             (.port right)
+            (ast:requires? (.port left))
             (ast:requires? (.port right))
-            (or (and
-                 (.instance.name left)
-                 (not (.instance.name right))
-                 (not (.external (.port left)))
-                 (.external (.port right)))
-                (and
-                 (not (.instance.name left))
-                 (.instance.name right)
-                 (.external (.port left))
-                 (not (.external (.port right))))))
+            (not (.external? (.port left)))
+            (.external? (.port right)))
            `(,(wfc-error o (format #f "cannot bind ~a port `~a' to ~a port `~a'"
-                                   (or (.external (.port left)) 'non-external) (.port.name left)
-                                   (or (.external (.port right)) 'non-external) (.port.name right)))
+                                   (or (.external? (.port left)) 'non-external) (.port.name left)
+                                   (or (.external? (.port right)) 'non-external) (.port.name right)))
              ,(wfc-info (.port left) (format #f "port `~a' defined here" (.port.name left)))
              ,(wfc-info (.port right) (format #f "port `~a' defined here" (.port.name right)))))
+          ((and
+            (.port left)
+            (.port right)
+            (not (.blocking? (.port left)))
+            (.blocking? (.port right)))
+           `(,(wfc-error o (format #f "cannot bind non-blocking port `~a' to blocking port `~a'"
+                                   (.port.name left) (.port.name right)))
+             ,(wfc-info (.port left) (format #f "non-blocking port `~a' defined here" (.port.name left)))
+             ,(wfc-info (.port right) (format #f "blocking port `~a' defined here" (.port.name right)))))
+          ((and
+            (.port left)
+            (.port right)
+            (ast:provides? (.port left))
+            (.blocking? (.port left))
+            (not (.blocking? (.port right))))
+           `(,(wfc-error (.port left) (format #f "superfluous blocking annotation on provides port `~a'"
+                                              (.port.name left)))))
           (else '()))))
 
 (define-method (binding-type (o <system>))
@@ -992,6 +1034,22 @@
           ,(wfc-info (.port right) (format #f "port `~a' defined here" (.port.name right))))
         '())))
 
+(define-method (blocking-ports (o <component-model>))
+  (let ((blocking-provides? (find .blocking? (ast:provides-port* o)))
+        (non-blocking-provides? (find (negate .blocking?) (ast:provides-port* o)))
+        (blocking-implementation? (or (model-blocking? o)
+                                      (find .blocking? (ast:requires-port* o)))))
+    (cond
+      ((and blocking-implementation? non-blocking-provides?)
+        `(,(wfc-error o "all provides ports should be defined as blocking")
+          ,(wfc-info non-blocking-provides?
+                    (format #f "non-blocking provides port `~a' define here"
+                            (.name non-blocking-provides?)))))
+      ((and (is-a? o <component>) (not blocking-implementation?) blocking-provides?)
+        `(,(wfc-error blocking-provides? (format #f "superfluous blocking annotation on provides port `~a'"
+                                                 (.name blocking-provides?)))))
+      (else '()))))
+
 (define-method (double-bindings (o <system>))
   (append-map double-bindings (ast:binding* o)))
 
@@ -1021,23 +1079,32 @@
         (else '())))
 
 (define-method (missing-bindings (o <system>))
-  (let* ((bindings (ast:binding* o))
-         (ports (append (filter-map (compose .port .left) bindings)
-                        (filter-map (compose .port .right) bindings)))
-         (ports (delete-duplicates ports ast:eq?)))
-    (append (append-map (cut missing-bindings <> o ports #f) (ast:port* o))
-            (append-map (cut missing-bindings <> o ports) (ast:instance* o)))))
-
-(define-method (missing-bindings (o <port>) (system <system>) ports instance)
-  (if (member o ports ast:eq?) '()
-      `(,(wfc-error o (format #f "port `~a' not bound" (.name o)))
-        ,@(if (not instance) '()
-              `(,(wfc-info instance (format #f "of instance: `~a'" (.name instance))))))))
-
-(define-method (missing-bindings (o <instance>) (system <system>) ports)
-  (append-map
-   (cute missing-bindings <> system ports o)
-   (filter (negate .injected) (if (not (.type o)) '() (ast:port* (.type o))))))
+  (define (binding->end-points binding)
+    `(,(.left binding) ,(.right binding)))
+  (define (port->end-point port)
+    (make <end-point> #:location (.location port) #:port.name (.name port)))
+  (define (instance->end-point instance)
+    (map (compose (cute make <end-point>
+                        #:location (.location instance)
+                        #:instance.name (.name instance)
+                        #:port.name <>)
+                  .name)
+         (filter (negate .injected?) (ast:port* (ast:type instance)))))
+  (define (end-point=? a b)
+    (and (equal? (.instance.name a) (.instance.name b))
+         (equal? (.port.name a) (.port.name b))))
+  (define (missing-end-point->wfc-error end-point)
+    (wfc-error (or (.instance end-point) (.port end-point))
+               (format #f "port `~a' of type `~a' not bound"
+                       (.port.name end-point)
+                       (type-name (.type.name (.port end-point))))))
+  (let* ((bound-end-points (append-map binding->end-points
+                                       (ast:binding* o)))
+         (end-points
+          (append (map port->end-point (ast:port* o))
+                  (append-map instance->end-point (ast:instance* o))))
+         (missing (lset-difference end-point=? end-points bound-end-points)))
+    (map missing-end-point->wfc-error missing)))
 
 (define-method (missing-return (o <function>))
   (define (function-body? s)
@@ -1100,32 +1167,32 @@
                (is-a? p <guard>)
                (is-a? p <on>)))
       `(,(wfc-error o (format #f "~a value discarded" class))))
-     ((and (not (is-a? (ast:type o) <void>))
-           (or (is-a? p <return>)
-               (is-a? grand-parent <return>)))
-      `(,(wfc-error o (format #f "~a used in a return expression" class))))
      ((and (not (is-a? (ast:type o) <void>))
            (not
             (or (is-a? p <assign>)
                 (is-a? p <variable>)
                 (is-a? p <reply>)
+                (is-a? p <return>)
                 (is-a? p <if>)))
            (not
             (and (or (is-a? p <equal>)
                      (is-a? p <not-equal>))
                  (or (is-a? grand-parent <if>)
-                     (is-a? grand-parent <reply>))
+                     (is-a? grand-parent <reply>)
+                     (is-a? grand-parent <return>))
                  (ast:eq? (.left p) o)))
            (not
             (and (or (is-a? p <and>)
                      (is-a? p <or>))
                  (or (is-a? grand-parent <if>)
-                     (is-a? grand-parent <reply>))
+                     (is-a? grand-parent <reply>)
+                     (is-a? grand-parent <return>))
                  (ast:eq? (.left p) o)))
            (not
             (and (is-a? p <not>)
                  (or (is-a? grand-parent <if>)
-                     (is-a? grand-parent <reply>))
+                     (is-a? grand-parent <reply>)
+                     (is-a? grand-parent <return>))
                  (ast:eq? (.expression p) o)))
            (not
             (and (or (is-a? p <equal>)
@@ -1133,7 +1200,8 @@
                  (or (is-a? grand-parent <and>)
                      (is-a? grand-parent <or>))
                  (or (is-a? great-grand-parent <if>)
-                     (is-a? great-grand-parent <reply>))
+                     (is-a? great-grand-parent <reply>)
+                     (is-a? great-grand-parent <return>))
                  (ast:eq? (.left grand-parent) p)))
            (not
             (and (is-a? p <not>)
@@ -1141,7 +1209,8 @@
                      (is-a? grand-parent <or>))
                  (let ((great-grand-parent (.parent grand-parent)))
                    (or (is-a? great-grand-parent <if>)
-                       (is-a? great-grand-parent <reply>)))
+                       (is-a? great-grand-parent <reply>)
+                       (is-a? great-grand-parent <return>)))
                  (ast:eq? (.left grand-parent) p))))
       `(,(wfc-error o (format #f "~a used in a complex expression" class))))
      (else '()))))
