@@ -1,7 +1,7 @@
 // dzn-runtime -- Dezyne runtime library
 //
-// Copyright © 2014, 2015, 2016, 2017, 2019, 2020 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
-// Copyright © 2015, 2016, 2017, 2019, 2020 Rutger van Beusekom <rutger@dezyne.org>
+// Copyright © 2014, 2015, 2016, 2017, 2019, 2020, 2021 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+// Copyright © 2015, 2016, 2017, 2019, 2020, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
 // Copyright © 2015 Paul Hoogendijk <paul@dezyne.org>
 //
 // This file is part of dzn-runtime.
@@ -24,6 +24,7 @@
 // Code:
 
 #include <dzn/runtime.hh>
+#include <dzn/coroutine.hh>
 
 #include <algorithm>
 #include <iostream>
@@ -31,59 +32,58 @@
 namespace dzn
 {
   std::ostream debug(nullptr);
-
   runtime::runtime(){}
 
   void trace(std::ostream& os, port::meta const& m, const char* e)
   {
-    os << path(m.require.meta, m.require.port) << "." << e << " -> "
-       << path(m.provide.meta, m.provide.port) << "." << e << std::endl;
+    os << path(m.require.meta, m.require.name) << "." << e << " -> "
+       << path(m.provide.meta, m.provide.name) << "." << e << std::endl;
   }
 
   void trace_out(std::ostream& os, port::meta const& m, const char* e)
   {
-    os << path(m.require.meta, m.require.port) << "." << e << " <- "
-       << path(m.provide.meta, m.provide.port) << "." << e << std::endl;
+    os << path(m.require.meta, m.require.name) << "." << e << " <- "
+       << path(m.provide.meta, m.provide.name) << "." << e << std::endl;
   }
 
   void trace_qin(std::ostream& os, port::meta const& m, const char* e)
   {
     if (path(m.provide.meta) == "<external>")
       os << path(m.require.meta, "<q>") << " <- "
-         << path(m.provide.meta, m.provide.port) << "." << e << std::endl;
+         << path(m.provide.meta, m.provide.name) << "." << e << std::endl;
     else
-      os <<  path(m.provide.meta, m.provide.port) << ".<q> <- "
-         <<  path(m.require.meta, m.require.port) << "." << e << std::endl;
+      os <<  path(m.provide.meta, m.provide.name) << ".<q> <- "
+         <<  path(m.require.meta, m.require.name) << "." << e << std::endl;
   }
 
   void trace_qout(std::ostream& os, port::meta const& m, const char* e)
   {
-    os << path(m.require.meta, m.require.port) << "." << e << " <- "
+    os << path(m.require.meta, m.require.name) << "." << e << " <- "
        << path(m.require.meta, "<q>") << std::endl;
   }
 
   bool runtime::external(void* scope) {
-    return (queues.find(scope) == queues.end());
+    return (states.find(scope) == states.end());
   }
 
-  bool& runtime::handling(void* scope)
+  size_t& runtime::handling(void* scope)
   {
-    return std::get<0>(queues[scope]);
+    return states[scope].handling;
   }
 
   void*& runtime::deferred(void* scope)
   {
-    return std::get<1>(queues[scope]);
+    return states[scope].deferred;
   }
 
   std::queue<std::function<void()> >& runtime::queue(void* scope)
   {
-    return std::get<2>(queues[scope]);
+    return states[scope].queue;
   }
 
   bool& runtime::performs_flush(void* scope)
   {
-    return std::get<3>(queues[scope]);
+    return states[scope].performs_flush;
   }
 
   bool& runtime::skip_block(void* port)
@@ -91,11 +91,9 @@ namespace dzn
     return skip_port[port];
   }
 
-  void runtime::flush(void* scope)
+  void runtime::flush(void* scope, size_t coroutine_id)
   {
-#ifdef DEBUG_RUNTIME
-    std::cout << path(reinterpret_cast<dzn::meta*>(scope)) << " flush" << std::endl;
-#endif
+    handling(scope) = 0;
     if(!external(scope))
     {
       std::queue<std::function<void()> >& q = queue(scope);
@@ -103,27 +101,25 @@ namespace dzn
       {
         std::function<void()> event = q.front();
         q.pop();
-        handle(scope, event);
+        handle(scope, event, coroutine_id);
+        handling(scope) = 0;
       }
       if (deferred(scope)) {
         void* tgt = deferred(scope);
         deferred(scope) = nullptr;
         if (!handling(tgt)) {
-          runtime::flush(tgt);
+          runtime::flush(tgt, coroutine_id);
         }
       }
     }
   }
 
-  void runtime::defer(void* src, void* tgt, const std::function<void()>& event)
+  void runtime::defer(void* src, void* tgt, const std::function<void()>& event, size_t coroutine_id)
   {
-#ifdef DEBUG_RUNTIME
-    std::cout << path(reinterpret_cast<dzn::meta*>(tgt)) << " defer" << std::endl;
-#endif
-
     if(!(src && performs_flush(src)) && !handling(tgt))
     {
-      handle(tgt, event);
+      handle(tgt, event, coroutine_id);
+      flush(tgt, coroutine_id);
     }
     else
     {
