@@ -1,8 +1,8 @@
 // dzn-runtime -- Dezyne runtime library
 //
-// Copyright © 2016, 2017, 2019 Jan Nieuwenhuizen <janneke@gnu.org>
+// Copyright © 2016, 2017, 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 // Copyright © 2017 Jvaneerd <J.vaneerd@student.fontys.nl>
-// Copyright © 2017, 2018, 2019, 2021 Rutger van Beusekom <rutger@dezyne.org>
+// Copyright © 2017, 2018, 2019, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
 // Copyright © 2016 Henk Katerberg <hank@mudball.nl>
 //
 // This file is part of dzn-runtime.
@@ -27,6 +27,7 @@
 using System;
 using System.Collections.Generic;
 using System.Diagnostics;
+using System.Linq;
 
 namespace dzn
 {
@@ -35,6 +36,20 @@ namespace dzn
         public RuntimeException(String msg) : base(msg) { }
     }
 
+    abstract public class Port
+    {
+        public dzn.port.Meta dzn_meta;
+        public Port()
+        {
+            dzn_meta = new dzn.port.Meta ();
+        }
+        public Port other ()
+        {
+            return (this == this.dzn_meta.provides.port)
+            ? this.dzn_meta.requires.port : this.dzn_meta.provides.port;
+        }
+    }
+
     abstract public class ComponentBase
     {
         public Locator dzn_locator;
@@ -52,7 +67,7 @@ namespace dzn
         public Component(Locator locator, String name = "", Meta parent = null)
             : base(locator, name, parent)
             {
-                this.dzn_runtime.infos.Add(this, new Runtime.info());
+                this.dzn_runtime.states.Add(this, new Runtime.State());
             }
     }
 
@@ -80,113 +95,121 @@ namespace dzn
     public class Runtime
     {
 
-        public class info
+        public class State
         {
-            public bool handling;
+            public int handling;
             public Component deferred;
             public Queue<Action> q;
             public bool flushes;
-            public info() { this.q = new Queue<Action>(); }
+            public State()
+            {
+                this.handling = 0;
+                this.q = new Queue<Action>();
+                this.flushes = false;
+            }
         }
 
-        public Dictionary<Object, info> infos;
+        public Dictionary<Object, State> states;
+        public Stack<Object> component_stack;
+        public Dictionary<int, Stack<Object>> blocked_port_component_stack;
         public Action illegal;
+
         public Runtime(Action illegal = null)
         {
             if (illegal == null)
-                {
-                    illegal = () => { throw new RuntimeException("illegal"); };
-                }
+            {
+               illegal = () => { throw new RuntimeException("illegal"); };
+            }
             this.illegal = illegal;
-            this.infos = new Dictionary<Object, info>();
+            this.states = new Dictionary<Object, State>();
+            this.component_stack = new Stack<Object>();
+            this.blocked_port_component_stack = new Dictionary<int, Stack<Object>>();
         }
         public bool external(Object c)
         {
-            return !infos.ContainsKey(c);
+            return !states.ContainsKey(c);
         }
-        public void flush(Object c)
+        public void flush(ComponentBase c)
         {
+          flush(c, coroutine_id(c.dzn_locator));
+        }
+        public void flush(Object c, int coroutine_id)
+        {
+            states[c].handling = 0;
             if (!external(c))
                 {
-                    while (infos[c].q.Count > 0)
+                    while (states[c].q.Count > 0)
                         {
-                            handle(c, infos[c].q.Dequeue());
+                            handle(c, states[c].q.Dequeue(), coroutine_id);
+                            states[c].handling = 0;
                         }
-                    if (infos[c].deferred != null)
+                    if (states[c].deferred != null)
                         {
-                            Component t = infos[c].deferred;
-                            infos[c].deferred = null;
-                            if (!infos[t].handling)
+                            Component t = states[c].deferred;
+                            states[c].deferred = null;
+                            if (states[t].handling == 0)
                                 {
-                                    flush(t);
+                                    flush(t, coroutine_id);
                                 }
                         }
                 }
         }
         public Queue<Action> queue(Object o)
         {
-            if(!infos.ContainsKey(o)) infos.Add(o, new Runtime.info());
-            return infos[o].q;
+            if(!states.ContainsKey(o)) states.Add(o, new Runtime.State());
+            return states[o].q;
         }
-        public void defer(Component i, Component o, Action f)
+        public void defer(Component src, Component tgt, Action f, int coroutine_id)
         {
-            if (!(i != null && infos[i].flushes) && !infos[o].handling)
+            if (!(src != null && states[src].flushes) && states[tgt].handling == 0)
                 {
-                    handle(o, f);
+                    handle(tgt, f, coroutine_id);
+                    flush(tgt, coroutine_id);
                 }
             else
                 {
-                    infos[o].q.Enqueue(f);
-                    if (i != null)
+                    states[tgt].q.Enqueue(f);
+                    if (src != null)
                         {
-                            infos[i].deferred = o;
+                            states[src].deferred = tgt;
                         }
                 }
         }
-        public R valued_helper<R>(Component c, Func<R> f) where R : struct, IComparable, IConvertible
+        public R valued_helper<R>(Component c, Func<R> f, int coroutine_id) where R : struct, IComparable, IConvertible
         {
-            if (infos[c].handling) throw new RuntimeException("a valued event cannot be deferred");
-
-            infos[c].handling = true;
-            R r = f();
-            infos[c].handling = false;
-            flush(c);
-            return r;
+            if (states[c].handling != 0) throw new RuntimeException("a valued event cannot be deferred");
+            states[c].handling = coroutine_id;
+            return f();
         }
-        public void handle(Object c, Action f)
+        public void handle(Object c, Action f, int coroutine_id)
         {
-            if (!infos[c].handling)
-                {
-                    infos[c].handling = true;
-                    f();
-                    infos[c].handling = false;
-                    flush(c);
-                }
-            else
-                {
-                    throw new RuntimeException("component already handling an event");
-                }
+            if (states[c].handling != 0) throw new RuntimeException("a valued event cannot be deferred");
+            states[c].handling = coroutine_id;
+            f();
         }
-        public void call_in<Port>(Component c, Action f, Port p, String e)
+        public void call_in(Component c, Action f, Port p, String e)
         {
-            if(infos[c].handling || dzn.pump.port_blocked_p(c.dzn_locator, p))
+            if(states[c].handling != 0 || dzn.pump.port_blocked_p(c.dzn_locator, p))
             {
-                dzn.pump.collateral_block(c.dzn_locator);
+              dzn.pump.collateral_block(c, c.dzn_locator);
             }
+            component_stack.Push(c);
             dzn.port.Meta m = (dzn.port.Meta) p.GetType().GetField("dzn_meta").GetValue(p);
             traceIn(m, e);
-            handle(c, f);
+            handle(c, f, coroutine_id(c.dzn_locator));
             traceOut(m, "return");
+            component_stack.Pop();
         }
-        public R call_in<R,Port>(Component c, Func<R> f, Port p, String e) where R : struct, IComparable, IConvertible
+        public R call_in<R>(Component c, Func<R> f, Port p, String e) where R : struct, IComparable, IConvertible
         {
-            if(infos[c].handling || dzn.pump.port_blocked_p(c.dzn_locator, p))
+            if(states[c].handling != 0 || dzn.pump.port_blocked_p(c.dzn_locator, p))
             {
-                dzn.pump.collateral_block(c.dzn_locator);
+              dzn.pump.collateral_block(c, c.dzn_locator);
             }
+            component_stack.Push(c);
             dzn.port.Meta m = (dzn.port.Meta) p.GetType().GetField("dzn_meta").GetValue(p);
             traceIn(m, e);
-            R r = valued_helper(c, f);
+            R r = valued_helper(c, f, coroutine_id(c.dzn_locator));
             String s;
             if (r.GetType().Equals(typeof(bool)))
                 s = (bool)Convert.ChangeType(r, typeof(bool)) ? "true" : "false";
@@ -195,13 +218,26 @@ namespace dzn
             else
                 s = r.GetType().Name + ":" + Enum.GetName(r.GetType(), r);
             traceOut(m, s);
+            component_stack.Pop();
             return r;
         }
-        public void call_out<Port>(Component c, Action f, Port p, String e)
+        public void call_out(Component c, Action f, Port p, String e)
         {
             dzn.port.Meta m = (dzn.port.Meta) p.GetType().GetField("dzn_meta").GetValue(p);
-            traceQin(m, e);
-            defer(m.provides.component, c, f);
+            if(!(p is async_base))
+              traceQin(m, e);
+            defer(m.provides.component, c, () => {
+              component_stack.Push(c);
+              if(!(p is async_base))
+                traceQout(m, e);
+              f();
+              component_stack.Pop();
+            }, coroutine_id(c.dzn_locator));
+        }
+        public static int coroutine_id(dzn.Locator l)
+        {
+          dzn.pump p = l.try_get<dzn.pump>();
+          return p == null ? 1 : p.coroutine_id();
         }
         public static String path(Meta m, String p = "")
         {
