@@ -1,8 +1,9 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
 ;;; Copyright © 2019 Timothy Sample <samplet@ngyro.com>
-;;; Copyright © 2019, 2020, 2021 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2021 Paul Hoogendijk <paul@dezyne.org>
+;;; Copyright © 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -155,6 +156,11 @@ output, and standard error as three values."
          (and=> (assq-ref alist flag)
                 (cute equal? <> '(#t))))))
 
+(define (queue-size file-name)
+  (let ((alist (get-meta file-name)))
+    (and alist
+         (and=> (assq-ref alist 'queue-size) car))))
+
 (define (flush? file-name)
   (get-meta-flag file-name 'flush))
 
@@ -191,11 +197,6 @@ output, and standard error as three values."
     (and alist
          (and=> (assq-ref alist 'non-strict?) car))))
 
-(define (no-interface-determinism? file-name)
-  (let ((alist (get-meta file-name)))
-    (and alist
-         (and=> (assq-ref alist 'no-interface-determinism?) car))))
-
 (define (error-model? file-name)
   (or (directory-exists? (string-append file-name "/baseline/verify"))
       ;; no verify baseline for system error models
@@ -248,13 +249,13 @@ output, and standard error as three values."
          (includes (filter directory-exists? includes))
          (includes (append-map (cut list "-I" <>) includes))
          (model (or (model? file-name) base-name))
-         (no-determinism? (no-interface-determinism? file-name))
+         (queue-size (queue-size file-name))
          (command
           `("dzn" "--verbose" "verify"
             ,@includes
             "--all"
+            ,@(if queue-size `("-q" ,(number->string queue-size)) '())
             ,@(if (model-unset? file-name) '() `("-m" ,model))
-            ,@(if no-determinism? '("--no-interface-determinism") `())
             ,dzn-name)))
     (or (skip? file-name "verify")
         (run-baseline file-name command
@@ -320,6 +321,7 @@ output, and standard error as three values."
          (includes (filter directory-exists? includes))
          (includes (append-map (cut list "-I" <>) includes))
          (model (or (model? file-name) base-name))
+         (queue-size (queue-size file-name))
          (out (string-append file-name "/out"))
          (out-lang (string-append file-name "/out/traces"))
          (handwritten-traces (list-files file-name ".*trace.*$")))
@@ -334,6 +336,7 @@ output, and standard error as three values."
                "-o" ,out-lang
                ,@(if (null? handwritten-traces) '("--traces") '())
                ,@(if (flush? file-name) '("--flush") '())
+               ,@(if queue-size `("-q" ,(number->string queue-size)) '())
                "--lts"
                ,dzn-name)
              input)
@@ -410,19 +413,19 @@ output, and standard error as three values."
          (out-lang (string-append file-name "/out/" language))
          (input (filter-<flush> input))
          (model (or (model? file-name) base-name))
+         (queue-size (queue-size file-name))
          ;; FIXME: METAs `model' is used for component/system tricksery
          (model base-name)
          (baseline? (file-exists? (string-append baseline "/" base-name)))
          (trace-format (cond ((trace-format file-name))
                              (baseline? "trace")
-                             (else "event")))
-         (no-determinism? (no-interface-determinism? file-name)))
+                             (else "event"))))
     (receive (status stdout stderr)
         (observe `("dzn" "simulate"
                    ,@includes
+                   ,@(if queue-size `("-q" ,(number->string queue-size)) '())
                    "--format" ,trace-format
                    ,@(if (non-strict? file-name) '() '("--strict"))
-                   ,@(if no-determinism? '("--no-interface-determinism") `())
                    "-m" ,model
                    ,dzn-name)
                  input)
@@ -445,11 +448,10 @@ output, and standard error as three values."
                          #f)))))
             (else
              (let ((out-file (string-append out-lang "/out"))
-                   (err-file (string-append out-lang "/err"))
-                   (trail (filter-state stdout)))
+                   (err-file (string-append out-lang "/err")))
                (mkdir-p out-lang)
                (with-output-to-file out-file
-                 (cut display trail))
+                 (cut display stdout))
                (with-output-to-file err-file
                  (cut display stderr))
                (let ((baseline-out (string-append baseline "/" base-name))
@@ -477,6 +479,7 @@ output, and standard error as three values."
                              '())))
          (includes (append-map (cut list "-I" <>) includes))
          (model (or (model? file-name) base-name))
+         (queue-size (queue-size file-name))
          (out (string-append file-name "/out"))
          (language "lts")
          (out-lang (string-append file-name "/out/" language))
@@ -490,11 +493,22 @@ output, and standard error as three values."
             (observe
              `("dzn" "graph" "--backend=lts"
                ,@includes
+               ,@(if queue-size `("-q" ,(number->string queue-size)) '())
                "-m" ,model
                ,dzn-name)
              input)
           (and (zero? status)
-               (let ((makreel-lts-file (string-append out "/traces/" base-name ".aut")))
+               (let* ((makreel-lts-file (string-append out "/traces/" base-name ".aut"))
+                      (lts-file (string-append out-lang "/" base-name ".aut"))
+                      (taus '("<deadlock>" "<livelock>" "<state>"))
+                      (modeling (list-matches
+                                 "\"([^\"]*[.](inevitable|optional))\""
+                                 stdout))
+                      (modeling (map (cute match:substring <> 1) modeling))
+                      (modeling (delete-duplicates modeling))
+                      (taus+modeling (append taus modeling)))
+                 (mkdir-p out-lang)
+                 (with-output-to-file lts-file (cute display stdout))
                  (or (and (not (file-exists? makreel-lts-file))
                           ;; XXX skip: "probably a system"
                           (format (current-error-port) "skip compare: ~s\n" makreel-lts-file))
@@ -502,19 +516,21 @@ output, and standard error as three values."
                             (flushes (list-matches "\"([^\"]*<flush>)\"" makreel-lts))
                             (flushes (map (cute match:substring <> 1) flushes))
                             (flushes (delete-duplicates flushes))
-                            (taus (append '("<deadlock>" "<livelock>" "optional" "inevitable" "<state>") flushes)))
+                            (blocking (list-matches "\"([^\"]*<blocking>)\"" makreel-lts))
+                            (blocking (map (cute match:substring <> 1) blocking))
+                            (blocking (delete-duplicates blocking))
+                            (modeling (append modeling '("inevitable" "optional")))
+                            (taus (append taus flushes blocking))
+                            (taus+modeling (append taus modeling)))
                        (with-output-to-file makreel-lts-file
                          (cute display makreel-lts))
-                       (mkdir-p out-lang)
-                       (with-output-to-file (string-append out-lang "/" base-name ".aut")
-                         (cute display stdout))
                        ;; FIXME: --structured-output with -eweak-trace still writes to disk
                        (with-directory-excursion out-lang
                          (receive (status stdout stderr)
                              (observe
                               `("ltscompare" "--counter-example" "--structured-output"
                                 "-eweak-trace"
-                                ,(string-append "--tau=" (string-join taus ","))
+                                ,(string-append "--tau=" (string-join taus+modeling ","))
                                 ,(string-append "../traces/" base-name ".aut")
                                 ,(string-append base-name ".aut"))
                               "")
