@@ -1,7 +1,7 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
 ;;; Copyright © 2020 Johri van Eerd <vaneerd.johri@gmail.com>
-;;; Copyright © 2020, 2021 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2020, 2021 Rutger van Beusekom <rutger@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
@@ -362,23 +362,23 @@
   (test-complete #:file-name "component-requires.dzn" #:line 13 #:column 11))
 
 (test-equal "component-requires requires"
-  '("external" "injected" "ihello")
+  '("blocking" "external" "injected" "ihello")
   (test-complete #:file-name "component-requires.dzn" #:line 14 #:column 11))
 
 (test-equal "component-requires external"
-  '("injected" "ihello")
+  '("blocking" "injected" "ihello")
   (test-complete #:file-name "component-requires.dzn" #:line 15 #:column 20))
 
 (test-equal "component-requires injected"
-  '("external" "ihello")
+  '("blocking" "external" "ihello")
   (test-complete #:file-name "component-requires.dzn" #:line 16 #:column 20))
 
 (test-equal "component-requires external injected"
-  '("ihello")
+  '("blocking" "ihello")
   (test-complete #:file-name "component-requires.dzn" #:line 17 #:column 29))
 
 (test-equal "component-requires injected external"
-  '("ihello")
+  '("blocking" "ihello")
   (test-complete #:file-name "component-requires.dzn" #:line 18 #:column 29))
 
 (test-equal "component-empty"
@@ -437,69 +437,70 @@
   '("provides" "requires")
   (test-complete #:file-name "typo.dzn" #:line 22 #:column 6))
 
+;;; component-{bool,data,enum,int}.dzn: Keep in sync.
 (test-equal "bool literal"
   '("false" "true")
-  (test-complete #:file-name "component-bool.dzn" #:line 10 #:column 13))
+  (test-complete #:file-name "component-bool.dzn" #:line 50 #:column 13))
 
 (test-equal "bool local"
   '("false" "fun(_)" "m" "true" "w.hello()")
-  (test-complete #:file-name "component-bool.dzn" #:line 36 #:column 15))
+  (test-complete #:file-name "component-bool.dzn" #:line 76 #:column 15))
 
 (test-equal "bool reply"
   '("b" "false" "fun(_)" "m" "true")
-  (test-complete #:file-name "component-bool.dzn" #:line 37 #:column 13))
+  (test-complete #:file-name "component-bool.dzn" #:line 77 #:column 13))
 
 (test-equal "bool formal"
   '("b" "false" "fun(_)" "m" "true")
-  (test-complete #:file-name "component-bool.dzn" #:line 31 #:column 13))
+  (test-complete #:file-name "component-bool.dzn" #:line 71 #:column 13))
 
 (test-equal "enum literal"
   '("Bool.False" "Bool.True")
-  (test-complete #:file-name "component-enum.dzn" #:line 10 #:column 13))
+  (test-complete #:file-name "component-enum.dzn" #:line 50 #:column 13))
 
 (test-equal "enum field"
   '("Bool.False" "Bool.True")
-  (test-complete #:file-name "component-enum.dzn" #:line 10 #:column 19))
+  (test-complete #:file-name "component-enum.dzn" #:line 50 #:column 19))
 
 (test-equal "enum local"
   '("Bool.False" "Bool.True" "fun(_)" "m" "w.hello()")
-  (test-complete #:file-name "component-enum.dzn" #:line 36 #:column 15))
+  (test-complete #:file-name "component-enum.dzn" #:line 76 #:column 15))
 
 (test-equal "enum reply"
   '("Bool.False" "Bool.True" "b" "fun(_)" "m")
-  (test-complete #:file-name "component-enum.dzn" #:line 37 #:column 13))
+  (test-complete #:file-name "component-enum.dzn" #:line 77 #:column 13))
 
 (test-equal "enum formal"
   '("Bool.False" "Bool.True" "b" "fun(_)" "m")
-  (test-complete #:file-name "component-enum.dzn" #:line 31 #:column 13))
+  (test-complete #:file-name "component-enum.dzn" #:line 71 #:column 13))
 
 (test-equal "int literal"
   '()
-  (test-complete #:file-name "component-int.dzn" #:line 10 #:column 13))
+  (test-complete #:file-name "component-int.dzn" #:line 50 #:column 13))
 
 (test-equal "int local"
   '("fun(_)" "m" "w.hello()")
-  (test-complete #:file-name "component-int.dzn" #:line 36 #:column 15))
+  (test-complete #:file-name "component-int.dzn" #:line 76 #:column 15))
 
 (test-equal "int reply"
   '("b" "fun(_)" "m")
-  (test-complete #:file-name "component-int.dzn" #:line 37 #:column 13))
+  (test-complete #:file-name "component-int.dzn" #:line 77 #:column 13))
 
 (test-equal "int formal"
   '("b" "fun(_)" "m")
-  (test-complete #:file-name "component-int.dzn" #:line 31 #:column 13))
+  (test-complete #:file-name "component-int.dzn" #:line 71 #:column 13))
 
 (test-equal "data literal"
   '()
-  (test-complete #:file-name "component-data.dzn" #:line 10 #:column 13))
+  (test-complete #:file-name "component-data.dzn" #:line 50 #:column 13))
 
 (test-equal "data local"
   '("m")
-  (test-complete #:file-name "component-data.dzn" #:line 36 #:column 15))
+  (test-complete #:file-name "component-data.dzn" #:line 76 #:column 15))
 
 (test-equal "data formal"
   '("b" "m")
-  (test-complete #:file-name "component-data.dzn" #:line 31 #:column 13))
+  (test-complete #:file-name "component-data.dzn" #:line 71 #:column 13))
 
 (test-equal "field-test field"
   '("e.False" "e.True")
@@ -510,7 +511,7 @@
   (test-complete #:file-name "enum.dzn" #:line 10 #:column 17))
 
 (test-equal "import requires"
-  '("external" "injected" "ihello" "ihello_enum" "ihello_int")
+  '("blocking" "external" "injected" "ihello" "ihello_enum" "ihello_int")
   (test-complete #:file-name "import.dzn" #:line 8 #:column 10
                  #:file-name->parse-tree file-name->parse-tree))
 
@@ -842,29 +843,30 @@
   "int.dzn:8:8"
   (test-lookup #:file-name "int.dzn" #:line 9 #:column 5))
 
+;;; component-{bool,data,enum,int}.dzn: Keep in sync.
 (test-equal "bool var->formal"
-  "component-bool.dzn:29:19"
-  (test-lookup #:file-name "component-bool.dzn" #:line 31 #:column 13))
+  "component-bool.dzn:69:19"
+  (test-lookup #:file-name "component-bool.dzn" #:line 71 #:column 13))
 
 (test-equal "enum var->formal"
-  "component-enum.dzn:29:19"
-  (test-lookup #:file-name "component-enum.dzn" #:line 31 #:column 13))
+  "component-enum.dzn:69:19"
+  (test-lookup #:file-name "component-enum.dzn" #:line 71 #:column 13))
 
 (test-equal "int var->formal"
-  "component-int.dzn:29:19"
-  (test-lookup #:file-name "component-int.dzn" #:line 31 #:column 13))
+  "component-int.dzn:69:19"
+  (test-lookup #:file-name "component-int.dzn" #:line 71 #:column 13))
 
 (test-equal "data var->formal"
-  "component-data.dzn:29:19"
-  (test-lookup #:file-name "component-data.dzn" #:line 31 #:column 13))
+  "component-data.dzn:69:19"
+  (test-lookup #:file-name "component-data.dzn" #:line 71 #:column 13))
 
 (test-equal "data var->formal"
-  "component-data.dzn:34:16"
-  (test-lookup #:file-name "component-data.dzn" #:line 37 #:column 13))
+  "component-data.dzn:74:16"
+  (test-lookup #:file-name "component-data.dzn" #:line 77 #:column 13))
 
 (test-equal "data arg->formal"
-  "component-data.dzn:34:16"
-  (test-lookup #:file-name "component-data.dzn" #:line 38 #:column 14))
+  "component-data.dzn:74:16"
+  (test-lookup #:file-name "component-data.dzn" #:line 78 #:column 14))
 
 (test-equal "variable->enum-type"
   "lookup.dzn:33:9"
